// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/driver/device.c $
// $Rev: 53222 $
// $Date: 2023-06-22 09:45:49 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model	Vendor	Device	SubVen	SubDev	type

	{ "16AIO",	0x10B5, 0x9080, 0x10B5, 0x2402,	GSC_DEV_TYPE_16AIO	},
	{ "12AIO",	0x10B5, 0x9080, 0x10B5, 0x2409,	GSC_DEV_TYPE_12AIO	},
	{ NULL }
};



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Do everything needed to setup and use the given device.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	u32	dma;
	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(AIO_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_bcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AIO_GSC_BCR));
		dev->vaddr.gsc_diopr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AIO_GSC_DIOPR));
		dev->vaddr.gsc_icr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AIO_GSC_ICR));
		dev->vaddr.gsc_idbr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AIO_GSC_IDBR));
		dev->vaddr.gsc_ibcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AIO_GSC_IBCR));
		dev->vaddr.gsc_odbr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AIO_GSC_ODBR));
		dev->vaddr.gsc_obcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AIO_GSC_OBCR));
		dev->vaddr.gsc_rgar_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AIO_GSC_RGAR));
		dev->vaddr.gsc_rgbr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AIO_GSC_RGBR));
		dev->vaddr.gsc_sscr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AIO_GSC_SSCR));

		// Data cache initialization.

		if (dev->board_type == GSC_DEV_TYPE_12AIO)
		{
			dev->cache.autocal_ms		= 3000;
			dev->cache.fgen_ai_max		= 1500000L;
			dev->cache.fgen_ai_min		= 458;
			dev->cache.fgen_ao_max		= 400000L;
			dev->cache.fgen_ao_min		= 458;
			dev->cache.fifo_size_rx		= _32K;
			dev->cache.fifo_size_tx		= _32K;
			dev->cache.initialize_ms	= 3;
			dev->cache.fref_default		= 30L * 1000L * 1000L;
			dev->cache.fsamp_ai_max		= 1500000L;
			dev->cache.fsamp_ai_min		= 458;
			dev->cache.fsamp_ao_max		= 400000L;
			dev->cache.fsamp_ao_min		= 458;
			dev->cache.nrate_ai_mask	= 0xFFFF;
			dev->cache.nrate_ai_max		= 0xFFFF;
			dev->cache.nrate_ai_min		= 20;
			dev->cache.nrate_ao_mask	= 0xFFFF;
			dev->cache.nrate_ao_max		= 0xFFFF;
			dev->cache.nrate_ao_min		= 2;
			dev->cache.res_bits			= 12;
		}
		else if (dev->board_type == GSC_DEV_TYPE_16AIO)
		{
			dev->cache.autocal_ms		= 3000;
			dev->cache.fgen_ai_max		= 300000L;
			dev->cache.fgen_ai_min		= 366;
			dev->cache.fgen_ao_max		= 300000L;
			dev->cache.fgen_ao_min		= 366;
			dev->cache.fifo_size_rx		= _32K;
			dev->cache.fifo_size_tx		= _32K;
			dev->cache.initialize_ms	= 3;
			dev->cache.fref_default		= 24L * 1000L * 1000L;
			dev->cache.fsamp_ai_max		= 300000L;
			dev->cache.fsamp_ai_min		= 366;
			dev->cache.fsamp_ao_max		= 300000L;
			dev->cache.fsamp_ao_min		= 366;
			dev->cache.nrate_ai_mask	= 0xFFFF;
			dev->cache.nrate_ai_max		= 0xFFFF;
			dev->cache.nrate_ai_min		= 2;
			dev->cache.nrate_ao_mask	= 0xFFFF;
			dev->cache.nrate_ao_max		= 0xFFFF;
			dev->cache.nrate_ao_min		= 80;
			dev->cache.res_bits			= 16;
		}
		else
		{
			ret	= -ENODEV;
#if DEV_PCI_ID_SHOW
			printk("This device is not supported by this driver.\n");
#endif
			break;
		}

		// Initialize additional resources.
		ret	= dev_irq_create(dev);
		if (ret)	break;

		ret	= dev_io_create(dev);
		if (ret)	break;

		dma	= GSC_DMA_SEL_STATIC
			| GSC_DMA_CAP_BMDMA_READ
			| GSC_DMA_CAP_BMDMA_WRITE;
		ret	= gsc_dma_create(dev, dma, dma);

		// Auto-detect the ICR IRQ bug.
		dev->cache.icr_irq_bug	= 0;
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ibcr_32, 0x880);
		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		gsc_dma_destroy(dev);
		dev_io_destroy(dev);
		dev_irq_destroy(dev);
		gsc_bar_destroy(&dev->bar);
		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);
		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


