// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/aoburst/perform.c $
// $Rev: 53244 $
// $Date: 2023-06-22 15:51:59 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	D11		(1 << 11)
#define	D18		(1 << 18)
#define	D19		(1 << 19)



// variables ******************************************************************

static	u32	_data[10240];
static	int	_samples;		// Number of samples in _data to send.
static	int	_scans;			// Number of scans in _data.



//*****************************************************************************
static int _init_data(const args_t* args)
{
	int		bs_max;
	int		chans	= 4;
	u32*	ptr		= _data;
	int		scan;
	u32		val;

	gsc_label("Initialize Data");
	memset(_data, 0, sizeof(_data));
	bs_max	= sizeof(_data) / 4 / chans;

	if (args->burst_size > bs_max)
		_scans	= bs_max;
	else
		_scans	= args->burst_size;

	_samples	= chans * _scans;

	for (scan = 0;;)
	{
		val		= (u32) ((float) 0xFFFF / _scans * scan);
		ptr[0]	= val | (0 << 16);
		ptr[1]	= val | (1 << 16);
		ptr[2]	= val | (2 << 16);
		ptr[3]	= val | (3 << 16) | D18;
		ptr		+= 4;
		scan	+= 1;

		if (scan == _scans)
		{
			ptr[-1]	|= D19;	// Burst End Flag
			break;
		}
	}

	printf("Done  (");
	gsc_label_long_comma(chans);
	printf(" channels, ");
	gsc_label_long_comma(_scans);
	printf(" scans, ");
	gsc_label_long_comma(chans * _scans);
	printf(" samples per burst)\n");
	return(0);
}



//*****************************************************************************
static int _config_ao(const args_t* args)
{
	int	b_src;
	int	errs	= 0;
	s32	master;
	int	nrate_a;
	int	nrate_b;
	s32	rgb_src;

	gsc_label("Device Configuration");
	printf("\n");
	gsc_label_level_inc();

	errs		+= aio_query(args->fd, -1, 0, AIO_QUERY_FREF_DEFAULT, &master);
	nrate_a		= master / args->sample_rate;
	nrate_b		= master / args->burst_rate;

	if (nrate_b <= 0xFFFF)
	{
		rgb_src	= AIO_RGB_CLK_SRC_MASTER;
	}
	else
	{
		rgb_src	= AIO_RGB_CLK_SRC_RGA;
		nrate_b	= master / (nrate_a * args->burst_rate);
	}

	if (args->manual)
		b_src	= AIO_AO_BURST_CLK_SRC_BCR;
	else
		b_src	= AIO_AO_BURST_CLK_SRC_RGB;

	errs	+= aio_initialize			(args->fd, -1, 1);
	errs	+= aio_tx_io_timeout		(args->fd, -1, 1, 30, NULL);
	errs	+= aio_range				(args->fd, -1, 1, AIO_RANGE_10V, NULL);
	errs	+= aio_data_format			(args->fd, -1, 1, AIO_DATA_FORMAT_OFF_BIN, NULL);
	errs	+= aio_ao_timing			(args->fd, -1, 1, AIO_AO_TIMING_SIMUL, NULL);
	errs	+= aio_ao_burst_enable		(args->fd, -1, 1, AIO_AO_BURST_ENABLE_YES, NULL);
	errs	+= aio_ao_looping			(args->fd, -1, 1, AIO_AO_LOOPING_DISABLE, NULL);
	errs	+= aio_ao_clk_src			(args->fd, -1, 1, AIO_AO_CLK_SRC_RGA, NULL);
	errs	+= aio_ao_burst_clk_src		(args->fd, -1, 1, b_src, NULL);
	errs	+= aio_ext_sync_src			(args->fd, -1, 1, AIO_AI_EXT_SYNC_SRC_AOS, NULL);
	errs	+= aio_rgb_clk_src			(args->fd, -1, 1, rgb_src, NULL);
	errs	+= aio_rga_nrate			(args->fd, -1, 1, nrate_a, NULL);
	errs	+= aio_rga_enable			(args->fd, -1, 1, AIO_GEN_ENABLE_YES, NULL);
	errs	+= aio_rgb_nrate			(args->fd, -1, 1, nrate_b, NULL);
	errs	+= aio_rgb_enable			(args->fd, -1, 1, AIO_GEN_ENABLE_YES, NULL);
	errs	+= aio_ao_buf_thr_lvl		(args->fd, -1, 1, _scans, NULL);
	errs	+= aio_irq0_sel				(args->fd, -1, 1, AIO_IRQ0_IDLE_INIT, NULL);
	errs	+= aio_irq1_sel				(args->fd, -1, 1, AIO_IRQ1_IDLE, NULL);
	errs	+= aio_irq2_sel				(args->fd, -1, 1, AIO_IRQ2_IDLE, NULL);
	errs	+= aio_fsamp_ao_report_all	(args->fd, -1, 1, NULL);
	errs	+= aio_fburst_ao_report		(args->fd, -1, 1, NULL);
	errs	+= aio_autocal				(args->fd, -1, 1);
	errs	+= aio_ao_buf_clear			(args->fd, -1, 1);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _prefill_buffer(const args_t* args)
{
	int	errs	= 0;
	s32	fifo	= 0;
	int	i;
	int	qty;
	int	send;
	int	sent;

	gsc_label("Prefill Output Buffer");
	printf("\n");
	gsc_label_level_inc();

	errs	+= aio_rgb_enable(args->fd, -1, 1, AIO_GEN_ENABLE_NO, NULL);
	errs	+= aio_tx_io_mode(args->fd, -1, 1, GSC_IO_MODE_PIO, NULL);

	gsc_label("Writing Data");
	errs	+= aio_query(args->fd, -1, 0, AIO_QUERY_FIFO_SIZE_TX, &fifo);
	qty		= fifo / (sizeof(_data) / 4);

	for (i = 0; i < qty; i++)
	{
		send	= _samples * 4;
		sent	= aio_write(args->fd, _data, send);

		if (sent < 0)
			break;

		if (sent != send)
		{
			errs++;
			printf("ERROR: aio_write() failure\n");
			break;
		}
	}

	printf("%s\n", errs ? "FAIL <---" : "PASS");

	if (args->io_mode != GSC_IO_MODE_PIO)
		errs	+= aio_tx_io_mode(args->fd, -1, 1, GSC_IO_MODE_BMDMA, NULL);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _io_write_auto(const args_t* args, int errs)
{
	struct timeval	end;
	struct timeval	now;
	int				send;
	int				sent;

	gsc_label("Burst Output");
	printf("(");
	gsc_label_long_comma(args->seconds);
	printf(" second%s)\n", (args->seconds == 1) ? "" : "s");
	gsc_label_level_inc();

	errs	+= aio_rgb_enable(args->fd, -1, 1, AIO_GEN_ENABLE_YES, NULL);

	gsc_label("Writing Data");

	if (errs)
	{
		printf("ABORTED DUE TO ERRORS.\n");
		errs	= 0;
	}
	else
	{
		gettimeofday(&end, NULL);
		end.tv_sec	+= args->seconds;
		errs		= 0;

		for (;;)
		{
			send	= _samples * 4;
			sent	= aio_write(args->fd, _data, send);

			if (sent < 0)
				break;

			if (sent != send)
			{
				errs++;
				printf("ERROR: aio_write() failure\n");
				break;
			}

			gettimeofday(&now, NULL);

			if (now.tv_sec < end.tv_sec)
				continue;

			if (now.tv_sec > end.tv_sec)
				break;

			if (now.tv_usec >= end.tv_usec)
				break;
		}

		if (errs == 0)
			printf("DONE\n");
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _io_write_manual(const args_t* args, int errs)
{
	u32				bcr;
	struct timeval	end;
	struct timeval	now;
	int				send;
	int				sent;
	struct timeval	wait;

	gsc_label("Burst Output");
	printf("(");
	gsc_label_long_comma(args->seconds);
	printf(" second%s)\n", (args->seconds == 1) ? "" : "s");
	gsc_label_level_inc();

	gsc_label("Writing Data");

	if (errs)
	{
		printf("ABORTED DUE TO ERRORS.\n");
		errs	= 0;
	}
	else
	{
		gettimeofday(&end, NULL);
		end.tv_sec	+= args->seconds;
		errs		= 0;

		for (;;)
		{
			// Write a data burst to the output buffer.
			send	= _samples * 4;
			sent	= aio_write(args->fd, _data, send);

			if (sent < 0)
				break;

			if (sent != send)
			{
				errs++;
				printf("ERROR: aio_write() failure\n");
				break;
			}

			// Issue a software trigger.
			errs	+= aio_ao_sync(args->fd, -1, 0);

			// Wait upto one second for the Burst Done status.
			gettimeofday(&wait, NULL);
			wait.tv_sec++;

			for (;;)
			{
				errs	+= aio_reg_read(args->fd, -1, 0, AIO_GSC_BCR, &bcr);

				if (errs)
					break;

				if ((bcr & D11) == 0)
					break;

				gettimeofday(&now, NULL);

				if (now.tv_sec < wait.tv_sec)
					continue;

				if (now.tv_sec > wait.tv_sec)
				{
					errs++;
					printf("FAIL <---  (Burst Done took more than one second.)\n");
					break;
				}

				if (now.tv_usec >= wait.tv_usec)
				{
					errs++;
					printf("FAIL <---  (Burst Done took more than one second.)\n");
					break;
				}
			}

			if (errs)
				break;

			gettimeofday(&now, NULL);

			if (now.tv_sec < end.tv_sec)
				continue;

			if (now.tv_sec > end.tv_sec)
				break;

			if (now.tv_usec >= end.tv_usec)
				break;
		}

		if (errs == 0)
			printf("DONE\n");
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;

	errs	+= _init_data(args);
	errs	+= _config_ao(args);

	if (args->manual)
	{
		errs	+= _io_write_manual(args, errs);
	}
	else
	{
		errs	+= _prefill_buffer(args);
		errs	+= _io_write_auto(args, errs);
	}

	return(errs);
}


