// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/auxout/perform.c $
// $Rev: 51294 $
// $Date: 2022-07-07 17:23:02 -0500 (Thu, 07 Jul 2022) $

// 16AIO: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	DIO_QTY		1

#ifdef CLOCK_MONOTONIC_RAW
#define	CLOCK		CLOCK_MONOTONIC_RAW
#else
#ifdef CLOCK_MONOTONIC
#define	CLOCK		CLOCK_MONOTONIC
#else
#define	CLOCK		CLOCK_REALTIME
#endif
#endif



//*****************************************************************************
#ifndef	CLOCK_MONOTONIC_RAW
#define	clock_gettime(c,ts)	_my_clock_gettime(ts)

static int _my_clock_gettime(struct timespec* ts)
{
	int				ret;
	struct timeval	tv;

	if (ts)
	{
		ret	= 0;
		gettimeofday(&tv, NULL);
		ts->tv_sec	= tv.tv_sec;
		ts->tv_nsec	= tv.tv_usec * 1000;
	}
	else
	{
		ret	= 1;
	}

	return(ret);
}
#endif



//*****************************************************************************
static int _config(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Configuration");
	printf("\n");
	gsc_label_level_inc();

	errs	+= aio_initialize(args->fd, -1, 1);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static void _output_header(void)
{
	printf("\n");
	printf("    Seq   Aux Out\n");
	printf("    No.   0        Time Stamp      Delta\n");
	printf("    ====  =======  ==============  ==============\n");
}



//*****************************************************************************
static int _dev_dio_out(const args_t* args, u16 value)
{
	static	int				count	= 0;
	static	struct timespec	t0;
	static	struct timespec	ts		= { 0, 0 };
	static	struct timespec	ts_last;

	u32						bit;
	long long				delta;
	int						errs	= 0;
	int						i;

	ts_last	= ts;
	clock_gettime(CLOCK, &ts);

	if (count == 0)
	{
		t0		= ts;
		ts_last	= ts;
	}

	errs	+= aio_aux_write(args->fd, -1, 0, value, NULL);

	count++;
	printf("    %-4d  ", count);

	for (i = DIO_QTY - 1; i >= 0; i--)
	{
		bit	= 0x1 << i;
		printf("%s", (value & bit) ? "1" : "0");

		if (bit & 0x1110)
			printf(" ");
	}

	printf("      ");

	delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
			- (1000000000LL * t0.tv_sec + t0.tv_nsec);
	printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

	delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
			- (1000000000LL * ts_last.tv_sec + ts_last.tv_nsec);
	printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

	printf("\n");
	fflush(stdout);

	if (args->ms_wait)
		os_sleep_ms(args->ms_wait);

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs;
	int	i;

	errs	= _config(args);

	gsc_label("Auxiliary Output");
	printf("\n");
	gsc_label_level_inc();

	_output_header();

	errs	+= _dev_dio_out(args, 0x0);
	errs	+= _dev_dio_out(args, 0x1);
	errs	+= _dev_dio_out(args, 0x0);
	errs	+= _dev_dio_out(args, 0x1);
	errs	+= _dev_dio_out(args, 0x0);

	for (i = 0; i < DIO_QTY; i++)
		errs	+= _dev_dio_out(args, 0x1 << i);

	errs	+= _dev_dio_out(args, 0x0);

	for (i = DIO_QTY - 1; i >= 0; i--)
		errs	+= _dev_dio_out(args, 0x1 << i);

	errs	+= _dev_dio_out(args, 0x0);
	errs	+= _dev_dio_out(args, 0x1);
	errs	+= _dev_dio_out(args, 0x0);
	errs	+= _dev_dio_out(args, 0x1);
	errs	+= _dev_dio_out(args, 0x0);

	gsc_label_level_inc();

	gsc_label("Status");
	printf("%s\n", errs ? "FAIL <---" : "PASS");

	gsc_label_level_dec();
	return(errs);
}



