// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/laout/perform.c $
// $Rev: 51298 $
// $Date: 2022-07-07 17:23:59 -0500 (Thu, 07 Jul 2022) $

// 16AIO: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32		_data[128 * 1024];
static	long	_qty;	// Number of samples to send per block.



//*****************************************************************************
static int _io_write(const args_t* args, int chans, int errs)
{
	char*	ptr;
	int		send;
	int		sent;
	long	tmp;
	u32		zero[4]	= { 0, 0, 0, 0 };

	gsc_label("Writing Data");

	for (;;)	// A convenience loop.
	{
		if (errs)
		{
			errs	= 0;
			printf("SKIPPED  (due to errors)\n");
			break;
		}

		errs	= 0;

		// Write the data to the device.
		send	= chans * 4;
		tmp		= aio_write(args->fd, zero, send);
		errs	+= (tmp != send) ? 1 : 0;

		send	= _qty * 4;
		ptr		= (char*) _data;

		for (;;)
		{
			sent	= aio_write(args->fd, ptr, send);

			if ((sent < 0) || (sent == send))
				break;

			ptr		+= sent;
			send	-= sent;
		}

		if (sent < 0)
			break;

		send	= chans * 4;
		tmp		= aio_write(args->fd, zero, send);
		errs	+= (tmp != send) ? 1 : 0;

		// Evaluate the results.

		if (sent < 0)
		{
			errs	= 1;
			printf("FAIL <---  (aio_write error)\n");
		}
		else
		{
			printf("PASS\n");
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
static int _init_data(const args_t* args, int chans)
{
	#define	PIE		(3.1415962)

	u32*		dest;
	int			errs;
	int			i;
	u32			max;
	long		period_len;	// The length of the pattern in samples.
	long		period_tot;	// Pattern samples total for all channels.
	long		period;		// Pattern length in x units.
	s32			res;
	int			samp;
	float		samp_dat;	// Total sample size of the data buffer.
	float		samp_sec;	// Samples Per Second for all active channels
	float		samp_tot;	// Samples Total for all active channels
	const char*	units;

	gsc_label("Initializing Data");
	errs	= aio_query(args->fd, -1, 0, AIO_QUERY_RES_BITS, &res);
	max		= (0x1UL << res) - 1;
	memset(_data, 0, sizeof(_data));
	samp_sec	= ((float) args->rate) * chans;
	samp_tot	= ((float) args->rate) * chans * args->seconds;
	samp_dat	= sizeof(_data) / 4;

	period_len	= (args->rate <= 100) ? args->rate				//      10-      100 ->    10-   100
				: (args->rate <= 1000) ? (args->rate / 2)		//     101-    1,000 ->    50-   500
				: (args->rate <= 10000) ? (args->rate / 5)		//   1,001-   10,000 ->   200- 2,000
				: (args->rate <= 100000) ? (args->rate / 10)	//  10,001-  100,000 -> 1,000-10,000
				: (args->rate / 100);							// 100,001-1,000,000 -> 1,000-10,000
	period_tot	= period_len * chans;
	period		= (long) (1000000000. / args->rate * period_len);
	units		= "ns";

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "us";
	}

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "ms";
	}

	if (samp_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get the entire image in the buffer.
		_qty	= (long) samp_tot;
	}
	else if (samp_sec <= SIZEOF_ARRAY(_data))
	{
		// We can get one second of the image in the buffer.
		_qty	= (long) samp_sec;
	}
	else if (period_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get one period of the image in the buffer.
		_qty	= period_tot;
	}
	else
	{
		// We can get only a fraction of a period of the image in the buffer.
		_qty	= (long) ((samp_dat / chans) * chans);
	}

	// Initialize the data buffer.
	// The data is encoded in OFFSET BINARY format.
	dest	= _data;;

	for (i = 0, samp = 0; i < _qty; i += chans, samp++)
	{
		samp	= (samp < period_len) ? samp : 0;

		// 1st channel: sawtooth wave
		*dest++	= ((u16) (max - (max * samp / (period_len - 1))))
				| 0x00000L;	// channel tag

		// 2nd channel: ramp wave
		*dest++	= ((u16) (max * samp / (period_len - 1)))
				| 0x10000L;	// channel tag

		// 3rd channel: square wave
		*dest++	= ((samp < (period_len / 2)) ? max : 0)
				| 0x20000L;	// channel tag

		// 4th channel: sine wave
		*dest++	= (u32) (((long) (max >> 1)) * (1 + sin((2 * PIE / period_len) * samp)))
				| 0x70000L;	// channel tag and group end flag
	}

	if (errs == 0)
		printf("DONE  (pattern length: %ld samples, %ld %s)\n", period_len, period, units);

	return(errs);
}



//*****************************************************************************
static int _perform_wait(const args_t* args, int errs)
{
	gsc_label("Waiting");

	if (errs)
	{
		printf("SKIPPED  (due to errors)\n");
	}
	else
	{
		gsc_time_sleep_ms(args->seconds * 1000);
		printf("Done  (%d second%s\n", (int) args->seconds, (args->seconds == 1) ? "" : "s");
	}

	return(0);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	chans	= 4;
	int	errs	= 0;

	gsc_label("Looping Analog Output");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _init_data		(args, chans);
	errs	+= aio_config_ao	(args->fd, -1, 1, args->rate);
	errs	+= aio_rgb_enable	(args->fd, -1, 1, AIO_GEN_ENABLE_NO, NULL);
	errs	+= _io_write		(args, chans, errs);
	errs	+= aio_ao_looping	(args->fd, -1, 1, AIO_AO_LOOPING_ENABLE, NULL);
	errs	+= aio_rgb_enable	(args->fd, -1, 1, AIO_GEN_ENABLE_YES, NULL);
	errs	+= _perform_wait	(args, errs);

	gsc_label_level_dec();
	return(errs);
}



