// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/savedata/perform.c $
// $Rev: 53249 $
// $Date: 2023-06-22 15:53:38 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M					(1024L * 1024L)
#define	TAG_DECODE(v)		(((v) & D16) ? 1 : 0)
#define	VALUE_DECODE(v)		((v) & 0xFFFF)
#define	D16					(0x10000)



// variables ******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _channels(const args_t* args, int* chans)
{
	int	errs	= 0;
	s32	get;
	s32	num;
	int	ret;

	gsc_label("Input Channels");

	get		= -1;
	ret		= aio_ioctl(args->fd, AIO_IOCTL_AI_SCAN_SIZE, &get);
	errs	+= ret ? 1 : 0;
	num		= -1;
	ret		= aio_ioctl(args->fd, AIO_IOCTL_AI_SCAN_1_CHAN, &num);
	errs	+= ret ? 1 : 0;

	switch (get)
	{
		default:

			errs++;
			chans[0]	= 16;
			printf(	"FAIL <---  (%d. unrecognixed option: %ld)\n",
					__LINE__,
					(long) get);
			break;

		case AIO_AI_SCAN_SIZE_SINGLE:

			chans[0]	= 1;
			printf("1 Channel (#%ld)\n", (long) num);
			break;

		case AIO_AI_SCAN_SIZE_0_1:

			chans[0]	= 2;
			printf("2 Channels (#0 to #1)\n");
			break;

		case AIO_AI_SCAN_SIZE_0_3:

			chans[0]	= 4;
			printf("4 Channel (#0 to #3)\n");
			break;

		case AIO_AI_SCAN_SIZE_0_7:

			chans[0]	= 8;
			printf("8 Channel (#0 to #7)\n");
			break;

		case AIO_AI_SCAN_SIZE_0_15:

			chans[0]	= 16;
			printf("16 Channel (#0 to #15)\n");
			break;

		case AIO_AI_SCAN_SIZE_0_31:

			chans[0]	= 32;
			printf("32 Channel (#0 to #21)\n");
			break;
	}

	return(errs);
}



//*****************************************************************************
static int _read_data(const args_t* args)
{
	char	buf[80];
	int		errs	= 0;
	long	get		= sizeof(_buffer);
	int		got;
	int		index;

	for (index = 0; (errs == 0) && (index < args->repeat);)
	{
		index++;
		strcpy(buf, "Reading");

		if (args->repeat > 1)
		{
			strcat(buf, " #");
			gsc_label_long_comma_buf(index, buf + strlen(buf));
		}

		gsc_label(buf);

		errs	= aio_ai_buf_clear(args->fd, -1, 0);

		if (errs)
		{
			printf("FAIL <---  (setup errors)\n");
			break;
		}

		got	= aio_read(args->fd, _buffer, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <---  (read error: %d)\n", got);
			break;
		}

		if (got != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (got %ld bytes, requested %ld)\n",
					(long) got,
					(long) get);
			break;
		}

		errs	= 0;
		printf("PASS  (");
		gsc_label_long_comma(get / 4);
		printf(" samples)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _save_eol(const char* name, FILE* file)
{
	int	errs	= 0;
	int	i;

	i	= fprintf(file, "\r\n");

	if (i != 2)
	{
		printf("FAIL <---  (fprintf() failure to %s)\n", name);
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _save_value(const args_t* args, const char* name, FILE* file, u32 value)
{
	char	buf1[40];
	char	buf2[80];
	int		errs	= 0;
	int		i;
	int		len;

	if (args->tag_strip)
		value = VALUE_DECODE(value);

	if (args->decimal)
		sprintf(buf1, "%ld", (long) value);
	else
		sprintf(buf1, "%08lX", (long) value);

	sprintf(buf2, "  %8s", buf1);
	len	= strlen(buf2);
	i	= fprintf(file, "%8s", buf2);

	if (i != len)
	{
		printf("FAIL <---  (fprintf() failure to %s)\n", name);
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _validate_data(int chans, u32 last, long index, u32 data)
{
	static	int	chan_tag	= -1;
	static	int	last_tag	= -1;

	int	errs				= 0;
	int	expect;

	if (data & 0xFFFE0000)
	{
		errs	= 1;
		printf(" FAIL <---  (Upper bits are invalid: [%ld] = 0x%08lX)\n", index, (long) data);
	}

	if (data & D16)			// Use D16 to compose channel number.
		chan_tag	= 0;
	else if (chan_tag >= 0)
		chan_tag++;

	if (last == 0xFFFFFFFF)
		last_tag	= (chan_tag + chans - 1) % chans;
	else if (last & D16)			// Use D16 to compose channel number.
		last_tag	= 0;
	else if (last_tag >= 0)
		last_tag++;

	if ((chan_tag >= 0) && (chan_tag >= chans))
	{
		errs	= 1;
		printf(" FAIL <---  (Invalid Channel Tag: [%ld] = 0x%08lX)\n", index, (long) data);
	}

	if (last == 0xFFFFFFFF)
	{
		if (chan_tag != 0)
		{
			errs	= 1;
			printf(" FAIL <---  (First tag not zero: [%ld] = 0x%08lX)\n", index, (long) data);
		}
	}
	else if (last_tag >= 0)
	{
		expect	= (last_tag + 1) % chans;

		if (chan_tag != expect)
		{
			errs	= 1;
			printf(	" FAIL <---  (Lost Synchronozation: [%ld] = 0x%08lX, [%ld] = 0x%08lX)\n",
					index - 1,
					(long) last,
					index,
					(long) data);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _save_data(const args_t* args, int chans, int errs)
{
	char		buf[64];
	FILE*		file;
	long		l;
	u32			last		= 0xFFFFFFFF;
	const char*	name		= "data.txt";
	long		samples		= sizeof(_buffer) / 4;
	int			tag_got;

	strcpy(buf, "Saving");

	if (args->force_save)
	{
		errs	= 0;
		strcat(buf, " (Forced)");
	}

	gsc_label(buf);

	for (;;)
	{
		if ((errs) && (args->force_save == 0))
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		for (l = 0; l < samples; l++)
		{
			tag_got	= TAG_DECODE(_buffer[l]);

			if ((l > 0) && (tag_got == 1))
				errs	+= _save_eol(name, file);

			errs	+= _save_value(args, name, file, _buffer[l]);
			errs	+= _validate_data(chans, last, l, _buffer[l]);
			last	= _buffer[l];
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	int	chans	= 32;

	gsc_label("Capture & Save");
	printf("\n");
	gsc_label_level_inc();

	errs	+= aio_config_ai	(args->fd, -1, 1, 15000);
	errs	+= aio_rx_io_mode	(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= _channels		(args, &chans);
	errs	+= aio_ai_buf_clear	(args->fd, -1, 1);
	errs	+= _read_data		(args);
	errs	+= _save_data		(args, chans, errs);

	gsc_label_level_dec();
	return(errs);
}



