// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/sbtest/main.h $
// $Rev: 53254 $
// $Date: 2023-06-22 15:56:25 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "16aio_main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)			(sizeof((a)) / sizeof((a)[0]))



// data types *****************************************************************

typedef struct
{
	// Application Settings

	s32	continuous;		// > 0 = ignore errors, < 0 = stop on errors
	s32	index;			// device index
	s32	minute_limit;	// for continuous testing
	int	qty;			// Number of devices detected.
	s32	test_limit;		// for continuous testing

	int	fd;				// File descriptor for the device to access.

	// Device Settings

} args_t;

typedef enum
{
	SERVICE_END_LIST,	// Ends list of service_data_t structures.
	SERVICE_NONE,		// Do nothing.
	SERVICE_NORMAL,
	SERVICE_REG_MOD,
	SERVICE_REG_READ,
	SERVICE_REG_SHOW,
	SERVICE_REG_TEST,
	SERVICE_REG_WRITE,	// arg = # of times to write
	SERVICE_SLEEP,		// arg = # of seconds to sleep
	SERVICE_SLEEP_MS,	// arg = # of milliseconds to sleep
	SERVICE_IOCTL_GET,
	SERVICE_IOCTL_SET
} service_t;

typedef struct
{
	service_t		service;
	unsigned long	cmd;	// IOCTL code
	s32				arg;	// The IOCTL data argument.
	u32				reg;	// The register to access. Use -1 to ignore.
	u32				mask;	// The register bits of interest.
	u32				value;	// The value expected for the bits of interest.
} service_data_t;



// prototypes *****************************************************************

int	ai_buf_clear_test(int fd);			// AIO_IOCTL_AI_BUF_CLEAR
int	ai_buf_thr_lvl_test(int fd);		// AIO_IOCTL_AI_BUF_THR_LVL
int	ai_buf_thr_sts_test(int fd);		// AIO_IOCTL_AI_BUF_THR_STS
int	ai_mode_test(int fd);				// AIO_IOCTL_AI_MODE
int	ai_scan_1_chan_test(int fd);		// AIO_IOCTL_AI_SCAN_1_CHAN
int	ai_scan_clk_src_test(int fd);		// AIO_IOCTL_AI_SCAN_CLK_SRC
int	ai_scan_size_test(int fd);			// AIO_IOCTL_AI_SCAN_SIZE
int	ai_sync_test(int fd);				// AIO_IOCTL_AI_SYNC
int	ao_buf_clear_test(int fd);			// AIO_IOCTL_AO_BUF_CLEAR
int	ao_buf_thr_lvl_test(int fd);		// AIO_IOCTL_AO_BUF_THR_LVL
int	ao_buf_thr_sts_test(int fd);		// AIO_IOCTL_AO_BUF_THR_STS
int	ao_burst_clk_src_test(int fd);		// AIO_IOCTL_AO_BURST_CLK_SRC
int	ao_burst_enable_test(int fd);		// AIO_IOCTL_AO_BURST_ENABLE
int	ao_clk_src_test(int fd);			// AIO_IOCTL_AO_CLK_SRC
int	ao_looping_test(int fd);			// AIO_IOCTL_AO_LOOPING
int	ao_sync_test(int fd);				// AIO_IOCTL_AO_SYNC
int	ao_timing_test(int fd);				// AIO_IOCTL_AO_TIMING
int	autocal_test(int fd);				// AIO_IOCTL_AUTOCAL
int	autocal_status_test(int fd);		// AIO_IOCTL_AUTOCAL_STATUS
int	aux_read_test(int fd);				// AIO_IOCTL_AUX_READ
int	aux_write_test(int fd);				// AIO_IOCTL_AUX_WRITE

int	data_format_test(int fd);			// AIO_IOCTL_DATA_FORMAT
int	dio_dir_out_test(int fd);			// AIO_IOCTL_DIO_DIR_OUT
int	dio_read_test(int fd);				// AIO_IOCTL_DIO_READ
int	dio_write_test(int fd);				// AIO_IOCTL_DIO_WRITE

int	ext_sync_src_test(int fd);			// AIO_IOCTL_EXT_SYNC_SRC

int	initialize_test(int fd);			// AIO_IOCTL_INITIALIZE

int	irq0_sel_test(int fd);				// AIO_IOCTL_IRQ0_SEL
int	irq1_sel_test(int fd);				// AIO_IOCTL_IRQ1_SEL
int	irq2_sel_test(int fd);				// AIO_IOCTL_IRQ2_SEL

int	perform_tests(const args_t* args);

int	query_test(int fd);					// AIO_IOCTL_QUERY

int	range_test(int fd);					// AIO_IOCTL_RANGE
int	reg_mod_test(int fd);				// AIO_IOCTL_REG_MOD
int	reg_read_test(int fd);				// AIO_IOCTL_REG_READ
int	reg_write_test(int fd);				// AIO_IOCTL_REG_WRITE
int	rga_enable_test(int fd);			// AIO_IOCTL_RGA_ENABLE
int	rga_nrate_test(int fd);				// AIO_IOCTL_RGA_NRATE
int	rgb_clk_src_test(int fd);			// AIO_IOCTL_RGB_CLK_SRC
int	rgb_enable_test(int fd);			// AIO_IOCTL_RGB_ENABLE
int	rgb_nrate_test(int fd);				// AIO_IOCTL_RGB_NRATE
int	rx_io_abort_test(int fd);			// AIO_IOCTL_RX_IO_ABORT
int	rx_io_mode_test(int fd);			// AIO_IOCTL_RX_IO_MODE
int	rx_io_timeout_test(int fd);			// AIO_IOCTL_RX_IO_TIMEOUT

int service_adjust_list_resolution	(int fd, service_data_t* list);
int	service_ioctl_reg_get_list		(int fd, int line, const service_data_t* list);
int	service_ioctl_set_get_list		(int fd, int line, const service_data_t* list);
int	service_ioctl_set_reg_list		(int fd, int line, const service_data_t* list);

int	tx_io_abort_test(int fd);			// AIO_IOCTL_TX_IO_ABORT
int	tx_io_mode_test(int fd);			// AIO_IOCTL_TX_IO_MODE
int	tx_io_timeout_test(int fd);			// AIO_IOCTL_TX_IO_TIMEOUT

int	wait_cancel_test(int fd);			// AIO_IOCTL_WAIT_CANCEL
int	wait_event_test(int fd);			// AIO_IOCTL_WAIT_EVENT
int	wait_status_test(int fd);			// AIO_IOCTL_WAIT_STATUS



#endif
