// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/sbtest/reg_write.c $
// $Rev: 51301 $
// $Date: 2022-07-07 17:24:53 -0500 (Thu, 07 Jul 2022) $

// 16AIO: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// This service is tested below.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
		u32	mask;
	} list[]	=
	{
		// reg			value		mask
		{ AIO_GSC_BCR,	0x777,		0x777	},
		{ AIO_GSC_BCR,	0x000,		0x777	},

		{ AIO_GSC_SSCR,	0x27FFF,	0x27FFF	},
		{ AIO_GSC_SSCR,	0x00000,	0x27FFF	},
	};

	int			errs	= 0;
	int			i;
	int			j;
	gsc_reg_t	parm;

	errs	+= aio_initialize(fd, -1, 0);

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= list[i].value;
		parm.mask	= 0xBEEFDEAD;
		j			= aio_ioctl(fd, AIO_IOCTL_REG_WRITE, (void*) &parm);

		if (j)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, aio_ioctl() failure)\n",
					__LINE__,
					i);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		// Now read the register and verify the bits of interest.
		j	= aio_ioctl(fd, AIO_IOCTL_REG_READ, (void*) &parm);

		if (j)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, aio_ioctl() failure)\n",
					__LINE__,
					i);
			break;
		}

		if ((list[i].value & list[i].mask) != (parm.value & list[i].mask))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, mask 0x%lX, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) list[i].mask,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	reg_write_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AIO_IOCTL_REG_WRITE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_write_test(int fd)
{
	int	errs	= 0;

	gsc_label("AIO_IOCTL_REG_WRITE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


