// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/sbtest/rx_io_timeout.c $
// $Rev: 53254 $
// $Date: 2023-06-22 15:56:25 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	AIO_IO_TIMEOUT_MAX,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	(AIO_IO_TIMEOUT_MIN + AIO_IO_TIMEOUT_MAX) / 2,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	AIO_IO_TIMEOUT_MIN,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	AIO_IO_TIMEOUT_NO_SLEEP,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	AIO_IO_TIMEOUT_DEFAULT,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= aio_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_get_list(fd, __LINE__, list);

	errs	+= service_ioctl_set_get_list(fd, __LINE__, list);

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	#define	TIEMOUT_1	2
	#define	TIEMOUT_2	6

	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	TIEMOUT_1,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	TIEMOUT_2,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32		data[1];
	long	dt1;
	long	dt2;
	int		errs	= 0;
	int		i;
	time_t	t1;
	time_t	t2;
	time_t	t3;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_get_list(fd, __LINE__, list_1);
	t1		= time(NULL);

	if (errs == 0)
	{
		i	= aio_read(fd, data, 4);

		if (i != 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. aio_read(): returned %d, expected 0)\n",
					__LINE__,
					i);
		}
	}

	errs	+= service_ioctl_set_get_list(fd, __LINE__, list_2);
	t2		= time(NULL);

	if (errs == 0)
	{
		i	= aio_read(fd, data, 4);

		if (i != 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. aio_read(): returned %d, expected 0)\n",
					__LINE__,
					i);
		}
	}

	t3	= time(NULL);
	dt1	= t2 - t1;
	dt2	= t3 - t2;

	if (errs)
	{
	}
	else if ((dt1 < (TIEMOUT_1 - 1)) || (dt1 > (TIEMOUT_1 + 1)))
	{
		errs++;
		printf(	"FAIL <---  (%d. timeout was %ld, expected %ld to %ld)\n",
				__LINE__,
				dt1,
				(long) TIEMOUT_1 - 1,
				(long) TIEMOUT_1 + 1);
	}
	else if ((dt2 < (TIEMOUT_2 - 1)) || (dt2 > (TIEMOUT_2 + 1)))
	{
		errs++;
		printf(	"FAIL <---  (%d. timeout was %ld, expected %ld to %ld)\n",
				__LINE__,
				dt2,
				(long) TIEMOUT_2 - 1,
				(long) TIEMOUT_2 + 1);
	}

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	rx_io_timeout_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AIO_IOCTL_RX_IO_TIMEOUT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_io_timeout_test(int fd)
{
	int	errs	= 0;

	gsc_label("AIO_IOCTL_RX_IO_TIMEOUT");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


