// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/sbtest/tx_io_abort.c $
// $Rev: 53254 $
// $Date: 2023-06-22 15:56:25 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	os_sem_t	sem_enter;
static	os_sem_t	sem_exit;



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _abort_none_test(int fd)
{
	int	errs	= 0;
	s32	get;
	int	ret;

	errs	+= aio_initialize(fd, -1, 0);
	ret		= aio_ioctl(fd, AIO_IOCTL_TX_IO_ABORT, &get);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (get))
	{
		errs++;
		printf(	"FAIL <---  (%d. got %ld, expected %ld)\n",
				__LINE__,
				(long) get,
				(long) 0);
	}

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _tx_thread(void* arg)
{
	char	buffer[1024];
	int		fd		= (int) (long) arg;

	os_sem_unlock(&sem_enter);
	memset(buffer, 0, sizeof(buffer));
	aio_write(fd, buffer, sizeof(buffer));
	os_sem_unlock(&sem_exit);

	return(0);
}



//*****************************************************************************
static int _abort_one_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_CLK_SRC,
			/* arg		*/	AIO_AO_CLK_SRC_DISABLE,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_TX_IO_TIMEOUT,
			/* arg		*/	15,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	32 * 10240,
			/* reg		*/	AIO_GSC_ODBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	s32			get;
	int			i;
	int			ret;
	os_thread_t	thread;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	os_sem_create_qty(&sem_enter, 1, 0);
	os_sem_create_qty(&sem_exit, 1, 0);
	memset(&thread, 0, sizeof(thread));
	errs	+= os_thread_create(
				&thread,
				"Tx I/O Abort",
				_tx_thread,
				(void*) (long) fd);

	// Wait for the thread to execute.
	os_sem_lock(&sem_enter);

	if (errs == 0)
	{
		for (i = 0; i < 20; )
		{
			ret		= aio_ioctl(fd, AIO_IOCTL_TX_IO_ABORT, &get);
			errs	+= ret ? 1 : 0;

			if ((errs) || (get))
				break;

			gsc_time_sleep_ms(1000);
		}

		if (errs)
		{
		}
		else if (get == 0)
		{
			errs++;
			printf("FAIL <---  (%d. abort not detected)\n", __LINE__);
		}
	}

	// Wait for the thread to finish.
	os_sem_lock(&sem_exit);

	// Finish up.
	errs	+= os_sem_destroy(&sem_enter);
	errs	+= os_sem_destroy(&sem_exit);
	errs	+= os_thread_destroy(&thread);
	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int errs	= 0;

	errs	+= _abort_none_test(fd);
	errs	+= _abort_one_test(fd);
	return(errs);
}



/******************************************************************************
*
*	Function:	tx_io_abort_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AIO_IOCTL_TX_IO_ABORT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int tx_io_abort_test(int fd)
{
	int	errs	= 0;

	gsc_label("AIO_IOCTL_TX_IO_ABORT");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


