// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/sbtest/wait_event.c $
// $Rev: 53254 $
// $Date: 2023-06-22 15:56:25 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1_SEC		(1000L)
#define	_15_SEC		(15L * 1000L)
#define	_30_SEC		(30L * 1000L)



// variables ******************************************************************

static	int			io_stat;
static	os_sem_t	sem_enter;
static	os_sem_t	sem_exit;
static	u32			u32_buf[4096];
static	gsc_wait_t	wait_dst;
static	int			wait_fd;



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no settings bits to test here.
	return(0);
}



//*****************************************************************************
static int _wait_main_dma(int fd)
{
	static const service_data_t	list[]	=
	{
		{					// Select DMA mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_BMDMA,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AI_SCAN_CLK_SRC,
			/* arg		*/	AIO_AI_SCAN_CLK_SRC_RGA,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RGA_ENABLE,
			/* arg		*/	AIO_GEN_ENABLE_YES,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	s32	sts		= 0;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

	if (errs == 0)
	{
		sts		= aio_read(fd, u32_buf, sizeof(u32_buf));
		errs	+= (sts == sizeof(u32_buf)) ? 0 : 1;
	}

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int	_wait_main_gsc(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_IRQ1_SEL,
			/* arg		*/	AIO_IRQ1_AI_BUF_THR_L2H,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	100,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AI_SCAN_CLK_SRC,
			/* arg		*/	AIO_AI_SCAN_CLK_SRC_RGA,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RGA_ENABLE,
			/* arg		*/	AIO_GEN_ENABLE_YES,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_idle_init(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_IRQ0_SEL,
			/* arg		*/	AIO_IRQ0_IDLE_INIT,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_autocal_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_IRQ0_SEL,
			/* arg		*/	AIO_IRQ0_AUTOCAL_DONE,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AUTOCAL,
			/* arg		*/	0,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_ai_buf_thr_l2h(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_IRQ1_SEL,
			/* arg		*/	AIO_IRQ1_AI_BUF_THR_L2H,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	100,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RGA_ENABLE,
			/* arg		*/	AIO_GEN_ENABLE_YES,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_ai_buf_thr_h2l(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_IRQ1_SEL,
			/* arg		*/	AIO_IRQ1_AI_BUF_THR_H2L,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	100,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RGA_ENABLE,
			/* arg		*/	AIO_GEN_ENABLE_YES,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RGA_ENABLE,
			/* arg		*/	AIO_GEN_ENABLE_NO,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_ao_buf_thr_l2h(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_IRQ2_SEL,
			/* arg		*/	AIO_IRQ2_AO_BUF_THR_L2H,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_BUF_THR_LVL,
			/* arg		*/	100,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_CLK_SRC,
			/* arg		*/	AIO_AO_CLK_SRC_DISABLE,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_BURST_CLK_SRC,
			/* arg		*/	AIO_AO_BURST_CLK_SRC_BCR,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	400,
			/* reg		*/	AIO_GSC_ODBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_ao_buf_thr_h2l(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_IRQ2_SEL,
			/* arg		*/	AIO_IRQ2_AO_BUF_THR_H2L,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_BUF_THR_LVL,
			/* arg		*/	100,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_CLK_SRC,
			/* arg		*/	AIO_AO_CLK_SRC_DISABLE,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_BURST_CLK_SRC,
			/* arg		*/	AIO_AO_BURST_CLK_SRC_BCR,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	400,
			/* reg		*/	AIO_GSC_ODBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_ao_burst_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_IRQ2_SEL,
			/* arg		*/	AIO_IRQ2_AO_BURST_DONE,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_BURST_CLK_SRC,
			/* arg		*/	AIO_AO_BURST_CLK_SRC_BCR,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_BURST_ENABLE,
			/* arg		*/	AIO_AO_BURST_ENABLE_YES,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_CLK_SRC,
			/* arg		*/	AIO_AO_CLK_SRC_RGA,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RGA_ENABLE,
			/* arg		*/	AIO_GEN_ENABLE_YES,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	64,
			/* reg		*/	AIO_GSC_ODBR,
			/* mask		*/	0,
			/* value	*/	0x80000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_SYNC,
			/* arg		*/	0,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int	_wait_io_rx_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RGA_ENABLE,
			/* arg		*/	AIO_GEN_ENABLE_YES,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[16];
	int	errs	= 0;
	s32	sts		= 0;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

	if (errs == 0)
	{
		sts		= aio_read(fd, buf, sizeof(buf));
		errs	+= (sts == sizeof(buf)) ? 0 : 1;
	}

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
#ifndef	INTIMEVER
static int	_wait_io_rx_error(int fd)
{
	int	errs	= 0;
	int	ret;

	errs	+= aio_initialize(fd, -1, 0);

	if (errs == 0)
	{
		ret	= aio_read(fd, NULL, 3);

		if (ret >= 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. expected %d, got %d)\n",
					__LINE__,
					-1,
					ret);
		}
	}

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}
#endif



//*****************************************************************************
static int	_wait_io_rx_timeout(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[32];
	int	errs	= 0;
	int	ret;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

	if (errs == 0)
	{
		ret	= aio_read(fd, buf, sizeof(buf));

		if (ret != 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. expected %d, got %d)\n",
					__LINE__,
					0,
					ret);
		}
	}

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _read_thread(void* arg)
{
	int		fd	= (int) (long) arg;
	long	status;

	status	= aio_read(fd, u32_buf, sizeof(u32_buf));

	if (status == 0)
	{
		io_stat	= 1;
	}
	else
	{
		io_stat	= 2;
		printf("FAIL <---  (%d. aio_read returned %ld, expected 0)\n",
				__LINE__,
				(long) status);
	}

	return(0);
}



//*****************************************************************************
static int	_wait_io_rx_abort(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	10,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	int			i;
	int			ret;
	s32			set;
	os_thread_t	thread;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

	// Now start the read thread so we can do the abort.
	memset(&thread, 0, sizeof(thread));

	if (errs == 0)
	{
		errs	+= os_thread_create(	&thread,
										"Rx",
										_read_thread,
										(void*) (long) fd);
	}

	for (i = 0; (errs == 0) && (i < 100); i++)
	{
		set		= 0;
		ret		= aio_ioctl(fd, AIO_IOCTL_RX_IO_ABORT, &set);
		errs	+= ret ? 1 : 0;

		if (errs)
			break;

		if (set)
			break;

		gsc_time_sleep_ms(100);
	}

	// Wait for the thread to report its status.

	for (i = 0; i < 100; i++)
	{
		if (errs)
			break;

		if (io_stat == 0)
		{
			gsc_time_sleep_ms(100);
			continue;
		}

		if (io_stat == 1)
			break;

		errs++;
		break;
	}

	errs	+= os_thread_destroy(&thread);
	errs	+= aio_initialize(fd, -1, 0);
	ret		= errs;
	return(ret);
}



//*****************************************************************************
static int	_wait_io_tx_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[32];
	int	errs	= 0;
	s32	sts;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

	if (errs == 0)
	{
		memset(buf, 0, sizeof(buf));
		sts		= aio_write(fd, buf, sizeof(buf));
		errs	+= (sts == sizeof(buf)) ? 0 : 1;
	}

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
#ifndef	INTIMEVER
static int	_wait_io_tx_error(int fd)
{
	int	errs	= 0;
	int	ret;

	errs	+= aio_initialize(fd, -1, 0);

	if (errs == 0)
	{
		ret	= aio_write(fd, NULL, 3);

		if (ret >= 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. expected %d, got %d)\n",
					__LINE__,
					-1,
					ret);
		}
	}

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}
#endif



//*****************************************************************************
static int	_wait_io_tx_timeout(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_CLK_SRC,
			/* arg		*/	AIO_AO_CLK_SRC_DISABLE,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_BURST_CLK_SRC,
			/* arg		*/	AIO_AO_BURST_CLK_SRC_BCR,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_TX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	34000,
			/* reg		*/	AIO_GSC_ODBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

	if (errs == 0)
	{
		memset(u32_buf, 0, sizeof(u32_buf));
		ret	= aio_write(fd, u32_buf, sizeof(u32_buf));

		if (ret != 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. expected %d, got %d)\n",
					__LINE__,
					0,
					ret);
		}
	}

	errs	+= aio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _write_thread(void* arg)
{
	int		fd	= (int) (long) arg;
	long	status;

	memset(u32_buf, 0, sizeof(u32_buf));
	status	= aio_write(fd, u32_buf, sizeof(u32_buf));

	if (status == 0)
	{
		io_stat	= 1;
	}
	else
	{
		io_stat	= 2;
		printf("FAIL <---  (%d. aio_write returned %ld, expected 0)\n",
				__LINE__,
				(long) status);
	}

	return(0);
}



//*****************************************************************************
static int	_wait_io_tx_abort(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_CLK_SRC,
			/* arg		*/	AIO_AO_CLK_SRC_DISABLE,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AIO_IOCTL_AO_BURST_CLK_SRC,
			/* arg		*/	AIO_AO_BURST_CLK_SRC_BCR,
			/* reg		*/	0,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	34000,
			/* reg		*/	AIO_GSC_ODBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	int			i;
	int			ret;
	s32			set;
	os_thread_t	thread;

	errs	+= aio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	memset(&thread, 0, sizeof(thread));

	if (errs == 0)
	{
		errs	+= os_thread_create(	&thread,
										"Tx",
										_write_thread,
										(void*) (long) fd);
	}

	for (i = 0; (errs == 0) && (i < 100); i++)
	{
		set		= 0;
		ret		= aio_ioctl(fd, AIO_IOCTL_TX_IO_ABORT, &set);
		errs	+= ret ? 1 : 0;

		if (errs)
			break;

		if (set)
			break;

		gsc_time_sleep_ms(100);
	}

	// Wait for the thread to report its status.

	for (i = 0; i < 100; i++)
	{
		if (errs)
			break;

		if (io_stat == 0)
		{
			gsc_time_sleep_ms(100);
			continue;
		}

		if (io_stat == 1)
			break;

		errs++;
		break;
	}

	errs	+= os_thread_destroy(&thread);
	errs	+= aio_initialize(fd, -1, 0);
	ret		= errs;
	return(ret);
}



//*****************************************************************************
static int	_wait_timeout(int fd)
{
	gsc_time_sleep_ms(2000);
	return(0);
}



//*****************************************************************************
static int	_wait_cancel(int fd)
{
	int			errs;
	int			ret;
	gsc_wait_t	wait;

	wait.flags			= 0;
	wait.main			= 0xFFFFFFFF;
	wait.gsc			= 0xFFFFFFFF;
	wait.alt			= 0xFFFFFFFF;
	wait.io				= 0xFFFFFFFF;
	wait.timeout_ms		= 0;
	wait.count			= 0;
	ret		= aio_ioctl(wait_fd, AIO_IOCTL_WAIT_CANCEL, &wait);
	errs	= ret ? 1 : 0;
	return(errs);
}



// variables ******************************************************************

#ifndef	INTIMEVER
// Under INtime for WIndows the induced error conditions are caught by the
// API Library and therefore don't make it to the driver. Thus, these tests
// always fail.
#define	_WAIT_IO_RX_ERROR	_wait_io_rx_error
#define	_WAIT_IO_TX_ERROR	_wait_io_tx_error
#else
#define	_WAIT_IO_RX_ERROR	NULL
#define	_WAIT_IO_TX_ERROR	NULL
#endif

static const struct
{
	const char*	name;
	int			(*function)(int fd);
	gsc_wait_t	wait;
} wait_src[]	=
{
	// name							function					flags			main							...			timeout_ms	count
	{ "DMA0/DMA1",					_wait_main_dma,				{ 0,			GSC_WAIT_MAIN_DMA0 |
																				GSC_WAIT_MAIN_DMA1,				0, 0, 0,	_15_SEC,	0	}	},
	{ "GSC",						_wait_main_gsc,				{ 0,			GSC_WAIT_MAIN_GSC,				0, 0, 0,	_15_SEC,	0	}	},

	// name							function					flags ...		gsc								...			timeout_ms	count
	{ "Initialize Done",			_wait_gsc_idle_init,		{ 0, 0,			AIO_WAIT_GSC_IDLE_INIT,			0, 0,		_15_SEC,	0	}	},
	{ "Autocal Done",				_wait_gsc_autocal_done,		{ 0, 0,			AIO_WAIT_GSC_AUTOCAL_DONE,		0, 0,		_15_SEC,	0	}	},
	{ "Aux Input Low->Hi",			NULL,						{ 0, 0,			AIO_WAIT_GSC_AUX_IN_L2H,		0, 0,		_15_SEC,	0	}	},
	{ "Aux Input Hi->Low",			NULL,						{ 0, 0,			AIO_WAIT_GSC_AUX_IN_H2L,		0, 0,		_15_SEC,	0	}	},

	{ "AI Buf Threshold Low->Hi",	_wait_gsc_ai_buf_thr_l2h,	{ 0, 0,			AIO_WAIT_GSC_AI_BUF_THR_L2H,	0, 0,		_15_SEC,	0	}	},
	{ "AI Buf Threshold Hi->Low",	_wait_gsc_ai_buf_thr_h2l,	{ 0, 0,			AIO_WAIT_GSC_AI_BUF_THR_H2L,	0, 0,		_15_SEC,	0	}	},

	{ "AO Buf Threshold Low->Hi",	_wait_gsc_ao_buf_thr_l2h,	{ 0, 0,			AIO_WAIT_GSC_AO_BUF_THR_L2H,	0, 0,		_15_SEC,	0	}	},
	{ "AO Buf Threshold Hi->Low",	_wait_gsc_ao_buf_thr_h2l,	{ 0, 0,			AIO_WAIT_GSC_AO_BUF_THR_H2L,	0, 0,		_15_SEC,	0	}	},
	{ "AO Burst Done",				_wait_gsc_ao_burst_done,	{ 0, 0,			AIO_WAIT_GSC_AO_BURST_DONE,		0, 0,		_15_SEC,	0	}	},

	// name							function					flags ...		io											timeout_ms	count
	{ "Rx Done",					_wait_io_rx_done,			{ 0, 0, 0, 0,	AIO_WAIT_IO_RX_DONE,						_15_SEC,	0	}	},
	{ "Rx Error",					_WAIT_IO_RX_ERROR,			{ 0, 0, 0, 0,	AIO_WAIT_IO_RX_ERROR,						_15_SEC,	0	}	},
	{ "Rx Timeout",					_wait_io_rx_timeout,		{ 0, 0, 0, 0,	AIO_WAIT_IO_RX_TIMEOUT,						_15_SEC,	0	}	},
	{ "Rx Abort",					_wait_io_rx_abort,			{ 0, 0, 0, 0,	AIO_WAIT_IO_RX_ABORT,						_15_SEC,	0	}	},
	{ "Tx Done",					_wait_io_tx_done,			{ 0, 0, 0, 0,	AIO_WAIT_IO_TX_DONE,						_15_SEC,	0	}	},
	{ "Tx Error",					_WAIT_IO_TX_ERROR,			{ 0, 0, 0, 0,	AIO_WAIT_IO_TX_ERROR,						_15_SEC,	0	}	},
	{ "Tx Timeout",					_wait_io_tx_timeout,		{ 0, 0, 0, 0,	AIO_WAIT_IO_TX_TIMEOUT,						_15_SEC,	0	}	},
	{ "Tx Abort",					_wait_io_tx_abort,			{ 0, 0, 0, 0,	AIO_WAIT_IO_TX_ABORT,						_15_SEC,	0	}	},

	// name							function					flags ...													timeout_ms	count
	{ "Wait Timeout",				_wait_timeout,				{ 0, 0, 0, 0,	AIO_WAIT_IO_RX_DONE,						_1_SEC,		0	}	},
	{ "Wait Cancel",				_wait_cancel,				{ 0, 0, 0, 0,	AIO_WAIT_IO_RX_DONE,						_30_SEC,	0	}	},

	// terminate list
	{ NULL, NULL, { 1, 0, 0, 0, 0, 0, 0 } }
};



//*****************************************************************************
static int _wait_thread(void* arg)
{
	int	status;

	os_sem_unlock(&sem_enter);
	status	= aio_ioctl(wait_fd, AIO_IOCTL_WAIT_EVENT, &wait_dst);

	if (status)
		wait_dst.flags	= 0xFFFFFFFF;

	os_sem_unlock(&sem_exit);
	return(0);
}


//*****************************************************************************
static int _process_thread(int index)
{
	char		buf[64];
	int			errs	= 0;
	int			i;
	int			ret;
	int			status;
	os_thread_t	thread;
	gsc_wait_t	wait;

	os_sem_create_qty(&sem_enter, 1, 0);
	os_sem_create_qty(&sem_exit, 1, 0);
	memset(&thread, 0, sizeof(thread));
	sprintf(buf, "wait event %d", index);
	wait_dst	= wait_src[index].wait;
	errs		+= os_thread_create(&thread, buf, _wait_thread, NULL);

	// Wait for the thread to become waiting.
	os_sem_lock(&sem_enter);

	if (errs == 0)
	{
		wait	= wait_src[index].wait;

		for (i = 0; i < 100; i++)
		{
			status		= aio_ioctl(wait_fd, AIO_IOCTL_WAIT_STATUS, &wait);

			if (status)
			{
				printf(	"FAIL <---  (%d. status request %d)\n",
						__LINE__,
						index);
				errs++;
				break;
			}

			if (wait.count == 1)
				break;

			gsc_time_sleep_ms(100);
		}
	}

	// Initiate the respective event.
	status	= wait_src[index].function(wait_fd);

	if (status < 0)
	{
	}
	else
	{
		errs	+= status;

		// Verify that the wait ended as expected.

		if (wait_src[index].wait.flags == 0xFFFFFFFF)
		{
		}
		else
		{
			// Wait for the resumed thread to run.
			os_sem_lock(&sem_exit);

			for (i = 0;; i++)
			{
				if (wait_dst.flags)
					break;

				if (i >= 100)
				{
					printf(	"FAIL <---  (%d. thread failed to resume)\n",
							__LINE__);
					errs++;
					break;
				}

				gsc_time_sleep_ms(100);
			}

			if (wait_src[index].wait.timeout_ms == _1_SEC)
			{
				if (wait_dst.flags != GSC_WAIT_FLAG_TIMEOUT)
				{
					printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
							__LINE__,
							(long) GSC_WAIT_FLAG_TIMEOUT,
							(long) wait_dst.flags);
					errs++;
				}
			}
			else if (wait_src[index].wait.timeout_ms == _15_SEC)
			{
				if (wait_dst.flags != GSC_WAIT_FLAG_DONE)
				{
					printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
							__LINE__,
							(long) GSC_WAIT_FLAG_DONE,
							(long) wait_dst.flags);
					errs++;
				}
			}
			else if (wait_src[index].wait.timeout_ms == _30_SEC)
			{
				if (wait_dst.flags != GSC_WAIT_FLAG_CANCEL)
				{
					printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
							__LINE__,
							(long) GSC_WAIT_FLAG_TIMEOUT,
							(long) wait_dst.flags);
					errs++;
				}
			}
			else
			{
				printf("FAIL <---  (%d. INTERNAL ERROR)\n", __LINE__);
				errs++;
			}
		}
	}

	wait.flags			= 0;
	wait.main			= 0xFFFFFFFF;
	wait.gsc			= 0xFFFFFFFF;
	wait.alt			= 0xFFFFFFFF;
	wait.io				= 0xFFFFFFFF;
	wait.timeout_ms		= 0;
	wait.count			= 0;
	ret		= aio_ioctl(wait_fd, AIO_IOCTL_WAIT_CANCEL, &wait);
	errs	+= ret ? 1 : 0;
	errs	+= os_sem_destroy(&sem_enter);
	errs	+= os_sem_destroy(&sem_exit);
	errs	+= os_thread_destroy(&thread);

	if ((status >= 0) && (errs == 0))
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;
	int	i;

	printf("\n");
	gsc_label_level_inc();

	// Create threads awaiting each of the source crtiteria
	// and verify that it resumes when the criteria is met.

	for (i = 0; wait_src[i].name; i++)
	{
		gsc_label(wait_src[i].name);

		wait_fd	= fd;
		memset(&wait_dst, 0, sizeof(wait_dst));

		if (wait_src[i].function)
			errs	+= _process_thread(i);
		else
			printf("SKIPPED\n");
	}

	gsc_label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	wait_event_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AIO_IOCTL_WAIT_EVENT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int wait_event_test(int fd)
{
	int	errs	= 0;

	gsc_label("AIO_IOCTL_WAIT_EVENT");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	return(errs);
}


