// $URL:$
// $Rev:$
// $Date:$

// 16AIO: Utilities Library: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio_autocal_status
*
*	Purpose:
*
*		Provide a visual wrapper for the AIO_IOCTL_AUTOCAL_STATUS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aio_autocal_status(int fd, int index, int verbose, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("Autocal Status", index);

	ret		= aio_ioctl(fd, AIO_IOCTL_AUTOCAL_STATUS, &set);
	errs	= ret ? 1 : 0;


	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AIO_AUTOCAL_STATUS_ACTIVE:

			ptr	= "Still In Progress";
			break;

		case AIO_AUTOCAL_STATUS_FAIL:

			ptr	= "FAIL";
			break;

		case AIO_AUTOCAL_STATUS_PASS:

			ptr	= "PASS";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


