// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/utils/util_config_ao.c $
// $Rev: 53259 $
// $Date: 2023-06-22 16:00:16 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Utilities Library: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio_config_ao
*
*	Purpose:
*
*		Configure the given device's analog output options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio_config_ao(int fd, int index, int verbose, s32 fsamp)
{
	int	errs	= 0;
	s32	nrate	= 0xFFFF;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= aio_tx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,			NULL);
	errs	+= aio_tx_io_timeout		(fd, index, verbose, 30,						NULL);

	errs	+= aio_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= aio_data_format			(fd, index, verbose, AIO_DATA_FORMAT_OFF_BIN,	NULL);

	errs	+= aio_ao_buf_thr_lvl		(fd, index, verbose, 16L * 1024,				NULL);
	errs	+= aio_ao_burst_enable		(fd, index, verbose, AIO_AO_BURST_ENABLE_NO,	NULL);
	errs	+= aio_ao_burst_clk_src		(fd, index, verbose, AIO_AO_BURST_CLK_SRC_BCR,	NULL);
	errs	+= aio_ao_looping			(fd, index, verbose, AIO_AO_LOOPING_DISABLE,	NULL);
	errs	+= aio_ao_timing			(fd, index, verbose, AIO_AO_TIMING_SIMUL,		NULL);
	errs	+= aio_range				(fd, index, verbose, AIO_RANGE_10V,				NULL);
	errs	+= aio_fsamp_ao_compute		(fd, index, verbose, fsamp, &nrate,				NULL);
	errs	+= aio_rgb_clk_src			(fd, index, verbose, AIO_RGB_CLK_SRC_MASTER,	NULL);
	errs	+= aio_rgb_enable			(fd, index, verbose, AIO_GEN_ENABLE_YES,		NULL);
	errs	+= aio_rgb_nrate			(fd, index, verbose, nrate,						NULL);
	errs	+= aio_ao_clk_src			(fd, index, verbose, AIO_AO_CLK_SRC_RGB,		NULL);

	errs	+= aio_fsamp_ao_report_all	(fd, index, verbose,							NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the device user manual for clarification.
//	errs	+= aio_autocal				(fd, index, verbose);	// Disconnect cable for autocal

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= aio_irq0_sel				(fd, index, verbose, AIO_IRQ0_IDLE_INIT,		NULL);
	errs	+= aio_irq1_sel				(fd, index, verbose, AIO_IRQ1_IDLE,				NULL);
	errs	+= aio_irq2_sel				(fd, index, verbose, AIO_IRQ2_IDLE,				NULL);

	// The final step is to clear the buffer.
	errs	+= aio_ao_buf_clear			(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


