// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/utils/util_fburst_ao.c $
// $Rev: 53259 $
// $Date: 2023-06-22 16:00:16 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Utilities Library: source file

#include "main.h"



//*****************************************************************************
static int _aio_fsamp_ao_get_rga(int fd, int verbose, s32* get)
{
	char	buf[256]	= "";
	s32		enable;
	int		errs		= 0;
	s32		master;
	s32		nrate;
	s32		set			= 0;

	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FREF_DEFAULT, &master);
	errs	+= aio_rga_enable(fd, -1, 0, -1, &enable);
	errs	+= aio_rga_nrate(fd, -1, 0, -1, &nrate);

	if (errs)
	{
	}
	else if (enable == AIO_GEN_ENABLE_NO)
	{
		gsc_label_long_comma_buf(set, buf);
		strcat(buf, " B/S  (Ref ");
		gsc_label_long_comma_buf(master, buf + strlen(buf));
		strcat(buf, " Hz, Gen A Disabled)\n");
	}
	else
	{
		set	= master / nrate;
		gsc_label_long_comma_buf(set, buf);
		strcat(buf, " B/S  (Ref ");
		gsc_label_long_comma_buf(master, buf + strlen(buf));
		strcat(buf, " Hz, Gen A, Nrate ");
		gsc_label_long_comma_buf(nrate, buf + strlen(buf));
		strcat(buf, ")\n");
	}

	if (verbose)
		printf("%s", buf);

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ao_get_rgb(int fd, int verbose, s32* get)
{
	s32		a_enable;
	s32		a_nrate;
	s32		b_enable;
	s32		b_nrate;
	s32		b_src;
	char	buf[256]	= "";
	int		errs		= 0;
	s32		master;
	s32		set			= 0;

	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FREF_DEFAULT, &master);
	errs	+= aio_rga_enable	(fd, -1, 0, -1, &a_enable);
	errs	+= aio_rga_nrate	(fd, -1, 0, -1, &a_nrate);
	errs	+= aio_rgb_enable	(fd, -1, 0, -1, &b_enable);
	errs	+= aio_rgb_nrate	(fd, -1, 0, -1, &b_nrate);
	errs	+= aio_rgb_clk_src	(fd, -1, 0, -1, &b_src);

	if (errs)
	{
	}
	else if (b_enable == AIO_GEN_ENABLE_NO)
	{
		gsc_label_long_comma_buf(set, buf + strlen(buf));
		strcat(buf," B/S  (Ref ");
		gsc_label_long_comma_buf(master, buf + strlen(buf));
		strcat(buf," Hz, Gen B Disabled)\n");
	}
	else if (b_src == AIO_RGB_CLK_SRC_MASTER)
	{
		set	= master / b_nrate;
		gsc_label_long_comma_buf(set, buf + strlen(buf));
		strcat(buf," B/S  (Ref ");
		gsc_label_long_comma_buf(master, buf + strlen(buf));
		strcat(buf," Hz, Gen B, Nrate ");
		gsc_label_long_comma_buf(b_nrate, buf + strlen(buf));
		strcat(buf,")\n");
	}
	else if (a_enable == AIO_GEN_ENABLE_NO)
	{
		gsc_label_long_comma_buf(set, buf + strlen(buf));
		strcat(buf," B/S  (Ref ");
		gsc_label_long_comma_buf(master, buf + strlen(buf));
		strcat(buf," Hz, Gen A Disabled)\n");
	}
	else
	{
		set	= master / (a_nrate * b_nrate);
		gsc_label_long_comma_buf(set, buf + strlen(buf));
		strcat(buf," B/S  (Ref ");
		gsc_label_long_comma_buf(master, buf + strlen(buf));
		strcat(buf," Hz, Gen A, Nrate ");
		gsc_label_long_comma_buf(a_nrate, buf + strlen(buf));
		strcat(buf,", Gen B, Nrate ");
		gsc_label_long_comma_buf(b_nrate, buf + strlen(buf));
		strcat(buf,")\n");
	}

	if (verbose)
		printf("%s", buf);

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ao_get_ext(int verbose, s32* get)
{
	char	buf[256]	= "";
	int		errs		= 0;
	s32		set			= 0;

	if (verbose)
	{
		gsc_label_long_comma_buf(set, buf + strlen(buf));
		strcat(buf, " B/S  (External Clock)\n");
	}

	if (verbose)
		printf("%s", buf);

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ao_get_bcr(int verbose, s32* get)
{
	int	errs	= 0;
	s32	set		= 0;

	if (verbose)
		printf("0 B/S  (SW Burst Triggering)\n");

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fburst_ao_get(int fd, int verbose, s32* get)
{
	int	errs;
	s32	set		= 0;
	s32	src;

	errs	= aio_ao_burst_clk_src(fd, -1, 0, -1, &src);

	if (errs == 0)
	{
		switch (src)
		{
			default:

				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (%d. unrecognized source: %ld)\n",
							__LINE__,
							(long) src);
				}

				break;

			case AIO_AO_BURST_CLK_SRC_RGA:	_aio_fsamp_ao_get_rga(fd, verbose, &set);	break;
			case AIO_AO_BURST_CLK_SRC_RGB:	_aio_fsamp_ao_get_rgb(fd, verbose, &set);	break;
			case AIO_AO_BURST_CLK_SRC_EXT:	_aio_fsamp_ao_get_ext(verbose, &set);		break;
			case AIO_AO_BURST_CLK_SRC_BCR:	_aio_fsamp_ao_get_bcr(verbose, &set);		break;
		}
	}

	if (get)
		get[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	aio_fburst_ao_report
*
*	Purpose:
*
*		Determine and report the output burst rate.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		get		Store the burst rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio_fburst_ao_report(int fd, int index, int verbose, s32* get)
{
	int	errs;

	if (verbose)
		gsc_label_index("AO Burst Rate", index);

	errs	= _aio_fburst_ao_get(fd, verbose, get);
	return(errs);
}


