// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/utils/util_fsamp_ai.c $
// $Rev: 53259 $
// $Date: 2023-06-22 16:00:16 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Utilities Library: source file

#include "main.h"



// macros *********************************************************************

#define	ABS(v)					(((v) < 0) ? (-(v)) : (v))
#define	LIMIT_MAX(v,max)		(((v) > (max)) ? (max) : (v))
#define	LIMIT_MIN(v,min)		(((v) < (min)) ? (min) : (v))
#define	LIMIT_RANGE(v,min,max)	(LIMIT_MIN((LIMIT_MAX((v),(max))),(min)))



//*****************************************************************************
static int _aio_fsamp_ai_get_mode_rga(int fd, int verbose, const char* test, int chan, s32* get)
{
	s32	enable;
	int	errs		= 0;
	s32	master;
	s32	nrate;
	s32	set			= 0;

	test	= test ? test : "";

	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FREF_DEFAULT, &master);
	errs	+= aio_rga_enable(fd, -1, 0, -1, &enable);
	errs	+= aio_rga_nrate(fd, -1, 0, -1, &nrate);

	if (errs)
	{
	}
	else if (enable == AIO_GEN_ENABLE_NO)
	{
		if (verbose)
		{
			gsc_label_long_comma(set);
			printf(" S/S  (%sChan %d, Ref ", test, chan);
			gsc_label_long_comma(master);
			printf(" Hz, Gen A Disabled)\n");
		}
	}
	else
	{
		set	= master / nrate;

		if (verbose)
		{
			gsc_label_long_comma(set);
			printf(" S/S  (%sChan %d, Ref ", test, chan);
			gsc_label_long_comma(master);
			printf(" Hz, Gen A, Nrate ");
			gsc_label_long_comma(nrate);
			printf(")\n");
		}
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_mode_rgb(int fd, int verbose, const char* test, int chan, s32* get)
{
	s32	a_enable;
	s32	a_nrate;
	s32	b_enable;
	s32	b_nrate;
	s32	b_src;
	int	errs		= 0;
	s32	master;
	s32	set			= 0;

	test	= test ? test : "";

	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FREF_DEFAULT, &master);
	errs	+= aio_rga_enable	(fd, -1, 0, -1, &a_enable);
	errs	+= aio_rga_nrate	(fd, -1, 0, -1, &a_nrate);
	errs	+= aio_rgb_enable	(fd, -1, 0, -1, &b_enable);
	errs	+= aio_rgb_nrate	(fd, -1, 0, -1, &b_nrate);
	errs	+= aio_rgb_clk_src	(fd, -1, 0, -1, &b_src);

	if (errs)
	{
	}
	else if (b_src == AIO_RGB_CLK_SRC_MASTER)
	{
		if (b_enable == AIO_GEN_ENABLE_NO)
		{
			if (verbose)
			{
				gsc_label_long_comma(set);
				printf(" S/S  (%sChan %d, Ref ", test, chan);
				gsc_label_long_comma(master);
				printf(" Hz, Gen B Disabled)\n");
			}
		}
		else
		{
			set	= master / b_nrate;

			if (verbose)
			{
				gsc_label_long_comma(set);
				printf(" S/S  (%sChan %d, Ref ", test, chan);
				gsc_label_long_comma(master);
				printf(" Hz, Gen B, Nrate ");
				gsc_label_long_comma(b_nrate);
				printf(")\n");
			}
		}
	}
	else if (a_enable == AIO_GEN_ENABLE_NO)
	{
		if (verbose)
		{
			gsc_label_long_comma(set);
			printf(" S/S  (%sChan %d, Ref ", test, chan);
			gsc_label_long_comma(master);
			printf(" Hz, Gen A Disabled)\n");
		}
	}
	else
	{
		set	= master / (a_nrate * b_nrate);

		if (verbose)
		{
			gsc_label_long_comma(set);
			printf(" S/S  (%sChan %d, Ref ", test, chan);
			gsc_label_long_comma(master);
			printf(" Hz, Gen A, Nrate ");
			gsc_label_long_comma(a_nrate);
			printf(", Gen B, Nrate ");
			gsc_label_long_comma(b_nrate);
			printf(")\n");
		}
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_mode_ext(int verbose, const char* test, int chan, s32* get)
{
	int	errs	= 0;
	s32	set		= 0;

	if (verbose)
	{
		test	= test ? test : "";
		gsc_label_long_comma(set);
		printf(" S/S  (%sChan %d, External Clock)\n", test, chan);
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_mode_bcr(int verbose, const char* test, int chan, s32* get)
{
	int	errs	= 0;
	s32	set		= 0;

	if (verbose)
	{
		test	= test ? test : "";
		gsc_label_long_comma(set);
		printf(" S/S  (%sChan %d, Software Sync)\n", test, chan);
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_mode(int fd, int verbose, int chan, s32* get)
{
	int	errs;
	s32	set		= 0;
	s32	src;

	errs	= aio_ai_scan_clk_src(fd, -1, 0, -1, &src);

	if (errs == 0)
	{
		switch (src)
		{
			default:

				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (%d. unrecognized source: %ld)\n",
							__LINE__,
							(long) src);
				}

				break;

			case AIO_AI_SCAN_CLK_SRC_RGA:	_aio_fsamp_ai_get_mode_rga(fd, verbose, "", chan, &set);	break;
			case AIO_AI_SCAN_CLK_SRC_RGB:	_aio_fsamp_ai_get_mode_rgb(fd, verbose, "", chan, &set);	break;
			case AIO_AI_SCAN_CLK_SRC_EXT:	_aio_fsamp_ai_get_mode_ext(verbose, "", chan, &set);		break;
			case AIO_AI_SCAN_CLK_SRC_BCR:	_aio_fsamp_ai_get_mode_bcr(verbose, "", chan, &set);		break;
		}
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_diff(int fd, int verbose, int chan, s32* get)
{
	s32	ch;
	int	errs	= 0;
	s32	max		= 0;
	s32	min		= 0;
	s32	set		= 0;
	s32	size;

	errs	+= aio_ai_scan_size(fd, -1, 0, -1, &size);
	errs	+= aio_ai_scan_1_chan(fd, -1, 0, -1, &ch);

	if (errs == 0)
	{
		switch (size)
		{
			default:

				errs++;

				if (verbose)
				{
					printf(	"FAIL <---  (%d. unrecognized size: %ld)\n",
							__LINE__,
							(long) size);
				}

				break;

			case AIO_AI_SCAN_SIZE_SINGLE:	min	= ch;	max	= ch;	break;
			case AIO_AI_SCAN_SIZE_0_1:		min	= 0;	max	= 1;	break;
			case AIO_AI_SCAN_SIZE_0_3:		min	= 0;	max	= 3;	break;
			case AIO_AI_SCAN_SIZE_0_7:		min	= 0;	max	= 7;	break;
			case AIO_AI_SCAN_SIZE_0_15:		min	= 0;	max	= 15;	break;
			case AIO_AI_SCAN_SIZE_0_31:		min	= 0;	max	= 31;	break;
		}

		if (errs)
		{
		}
		else if ((chan >= min) && (chan <= max))
		{
			errs	+= _aio_fsamp_ai_get_mode(fd, verbose, chan, &set);
		}
		else if (verbose)
		{
			gsc_label_long_comma(set);
			printf(" S/S  (Chan %d, invalid channel)\n", chan);
		}
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_single(int fd, int verbose, int chan, s32* get)
{
	s32	ch;
	int	errs;
	s32	set		= 0;

	errs	= aio_ai_scan_1_chan(fd, -1, 0, -1, &ch);

	if (errs)
	{
	}
	else if (chan == ch)
	{
		errs	= _aio_fsamp_ai_get_mode(fd, verbose, chan, &set);
	}
	else if (verbose)
	{
		gsc_label_long_comma(set);
		printf(" S/S  (Chan %d, invalid channel)\n", chan);
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_test(int fd, int verbose, const char* name, int chan, s32* get)
{
	char	buf[128]	= "";
	int		errs;
	s32		set			= 0;
	s32		src;

	errs	= aio_ai_scan_clk_src(fd, -1, 0, -1, &src);

	if (errs)
	{
	}
	else if (chan == 0)
	{
		if (verbose)
			sprintf(buf, "%s Test Mode, ", name);

		switch (src)
		{
			default:

				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (%d. unrecognized source: %ld)\n",
							__LINE__,
							(long) src);
				}

				break;

			case AIO_AI_SCAN_CLK_SRC_RGA:	errs	= _aio_fsamp_ai_get_mode_rga(fd, verbose, buf, chan, get);	break;
			case AIO_AI_SCAN_CLK_SRC_RGB:	errs	= _aio_fsamp_ai_get_mode_rgb(fd, verbose, buf, chan, get);	break;
			case AIO_AI_SCAN_CLK_SRC_EXT:	errs	= _aio_fsamp_ai_get_mode_ext(verbose, buf, chan, get);		break;
			case AIO_AI_SCAN_CLK_SRC_BCR:	errs	= _aio_fsamp_ai_get_mode_bcr(verbose, buf, chan, get);		break;
		}
	}
	else if (verbose)
	{
		gsc_label_long_comma(set);
		printf(" S/S  (%s Test Mode, Chan %d, invalid channel)\n", name, chan);
	}

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_zero(int fd, int verbose, int chan, s32* get)
{
	int	errs;

	errs	= _aio_fsamp_ai_get_test(fd, verbose, "Zero Volt", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_vref(int fd, int verbose, int chan, s32* get)
{
	int	errs;

	errs	= _aio_fsamp_ai_get_test(fd, verbose, "Vref", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_ao_0(int fd, int verbose, int chan, s32* get)
{
	int	errs;

	errs	= _aio_fsamp_ai_get_test(fd, verbose, "AO Chan 0", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_ao_1(int fd, int verbose, int chan, s32* get)
{
	int	errs;

	errs	= _aio_fsamp_ai_get_test(fd, verbose, "AO Chan 1", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_ao_2(int fd, int verbose, int chan, s32* get)
{
	int	errs;

	errs	= _aio_fsamp_ai_get_test(fd, verbose, "AO Chan 2", chan, get);
	return(errs);
}



//*****************************************************************************
static int _aio_fsamp_ai_get_ao_3(int fd, int verbose, int chan, s32* get)
{
	int	errs;

	errs	= _aio_fsamp_ai_get_test(fd, verbose, "AO Chan 3", chan, get);
	return(errs);
}



//*****************************************************************************
int aio_fsamp_ai_compute(int fd, int index, int verbose, s32 fsamp, s32* nrate, double* rate)
{
	double	delta;
	double	delta_best	= FLT_MAX;
	int		errs		= 0;
	double	fsamp_best	= 0;
	double	fsamp_got;
	s32		fsamp_max;	// Queried value.
	s32		fsamp_min;	// Queried value.
	double	fsamp_want;
	int		init		= 0;	// Have we performed the first run initialization?
	s32		master;
	s32		nr;
	s32		nr_max;
	s32		nr_min;
	s32		nr_use;
	s32		nrate_best	= 0;
	s32		nrate_max;	// Queried value.
	s32		nrate_min;	// Queried value.

	if (verbose)
	{
		gsc_label_index("AI Sample Rate", index);
		gsc_label_long_comma(fsamp);
		printf(" S/S");
	}

	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FSAMP_AI_MAX, &fsamp_max);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FSAMP_AI_MIN, &fsamp_min);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FREF_DEFAULT, &master);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_NRATE_AI_MAX, &nrate_max);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_NRATE_AI_MIN, &nrate_min);

	fsamp_want	= (double) LIMIT_RANGE(fsamp, fsamp_min, fsamp_max);
	nr			= (s32) (master / fsamp_want);
	nr			= LIMIT_RANGE(nr, nrate_min, nrate_max);
	nr_min		= nr - 1;
	nr_max		= nr + 1;
	nr_min		= (nr_min < nrate_min) ? nrate_min : nr_min;
	nr_max		= (nr_max > nrate_max) ? nrate_max : nr_max;
	nrate[0]	= nr;

	for (nr = nr_min; nr <= nr_max; nr++)
	{
		if (nr < nr_min)
			nr_use	= nr_min;
		else if (nr > nr_max)
			nr_use	= nr_max;
		else
			nr_use	= nr;

		fsamp_got	= ((double) master) / nr_use;
		delta		= fsamp_got - fsamp_want;

		if ((init == 0) || ((ABS(delta)) < (ABS(delta_best))))
		{
			init		= 1;
			delta_best	= delta;
			fsamp_best	= fsamp_got;
			nrate_best	= nr_use;
		}
	}

	nrate[0]	= nrate_best;

	if (rate)
		rate[0]		= fsamp_best;

	if (verbose)
	{
		printf("%s  (", errs ? " FAIL <---" : "");
		printf("Fref ");
		gsc_label_long_comma((long) master);
		printf(", Nrate ");
		gsc_label_long_comma((long) nrate_best);
		printf(", Fsamp ");
		gsc_label_float_comma(fsamp_best, -1, 3);
		printf(")\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	aio_fsamp_ai_get
*
*	Purpose:
*
*		Determine the input sample rate.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		chan	The channel for which the request is being made.
*
*		get		Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio_fsamp_ai_get(int fd, int index, int verbose, int chan, s32* get)
{
	int	errs;
	s32	mode;
	s32	set		= 0;

	if (verbose)
		gsc_label_index("AI Sample Rate", index);

	errs	= aio_ai_mode(fd, -1, 0, -1, &mode);

	if (errs == 0)
	{
		switch (mode)
		{
			default:

				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (%d. unrecognized mode: %ld)\n",
							__LINE__,
							(long) mode);
				}

				break;

			case AIO_AI_MODE_DIFF:		errs	+= _aio_fsamp_ai_get_diff	(fd, verbose, chan, &set);	break;
			case AIO_AI_MODE_SINGLE:	errs	+= _aio_fsamp_ai_get_single	(fd, verbose, chan, &set);	break;
			case AIO_AI_MODE_ZERO:		errs	+= _aio_fsamp_ai_get_zero	(fd, verbose, chan, &set);	break;
			case AIO_AI_MODE_VREF:		errs	+= _aio_fsamp_ai_get_vref	(fd, verbose, chan, &set);	break;
			case AIO_AI_MODE_AO_0:		errs	+= _aio_fsamp_ai_get_ao_0	(fd, verbose, chan, &set);	break;
			case AIO_AI_MODE_AO_1:		errs	+= _aio_fsamp_ai_get_ao_1	(fd, verbose, chan, &set);	break;
			case AIO_AI_MODE_AO_2:		errs	+= _aio_fsamp_ai_get_ao_2	(fd, verbose, chan, &set);	break;
			case AIO_AI_MODE_AO_3:		errs	+= _aio_fsamp_ai_get_ao_3	(fd, verbose, chan, &set);	break;
		}
	}


	if (get)
		get[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	aio_fsamp_ai_report
*
*	Purpose:
*
*		Determine and report the input sample rate for the specified channel.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		chan	The index of the channel of interest.
*
*		get		Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio_fsamp_ai_report(int fd, int index, int verbose, int chan, s32* get)
{
	char	buf[64];
	int		errs;
	s32		set		= 0;

	if (verbose)
	{
		sprintf(buf, "AI Channel %d", chan);
		gsc_label_index(buf, index);
	}

	errs	= aio_fsamp_ai_get(fd, index, 0, chan, &set);

	if (verbose)
	{
		gsc_label_long_comma(set);
		printf(" S/S\n");
	}

	if (get)
		get[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	aio_fsamp_ai_report_all
*
*	Purpose:
*
*		Determine and report the input sample rate for all channels.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The overall sample rate is reported here.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio_fsamp_ai_report_all(int fd, int index, int verbose, s32* fsamp)
{
	s32	chan;
	int	errs	= 0;
	int	i;
	int	max;
	int	min;
	s32	mode;
	s32	rate;
	s32	size;
	s32	total	= 0;;

	if (verbose)
	{
		gsc_label_index("AI Sample Rates", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= aio_ai_mode(fd, -1, 0, -1, &mode);
	errs	+= aio_ai_scan_size(fd, -1, 0, -1, &size);
	errs	+= aio_ai_scan_1_chan(fd, -1, 0, -1, &chan);

	if (errs == 0)
	{
		switch (size)
		{
			default:
			case AIO_AI_SCAN_SIZE_SINGLE:	min	= chan;	max	= chan;	break;
			case AIO_AI_SCAN_SIZE_0_1:		min	= 0;	max	= 1;	break;
			case AIO_AI_SCAN_SIZE_0_3:		min	= 0;	max	= 3;	break;
			case AIO_AI_SCAN_SIZE_0_7:		min	= 0;	max	= 7;	break;
			case AIO_AI_SCAN_SIZE_0_15:		min	= 0;	max	= 15;	break;
			case AIO_AI_SCAN_SIZE_0_31:		min	= 0;	max	= 31;	break;
		}

		switch (mode)
		{
			case AIO_AI_MODE_ZERO:
			case AIO_AI_MODE_VREF:
			case AIO_AI_MODE_AO_0:
			case AIO_AI_MODE_AO_1:
			case AIO_AI_MODE_AO_2:
			case AIO_AI_MODE_AO_3:

				if (min > 0)
				{
					rate	= 0;
					errs	+= aio_fsamp_ai_report(fd, -1, verbose, 0, &rate);
					total	+= rate;
				}

				break;

			default:

				break;
		}

		for (i = min; i <= max; i++)
		{
			rate	= 0;
			errs	+= aio_fsamp_ai_report(fd, -1, verbose, i, &rate);
			total	+= rate;
		}

		if (verbose)
		{
			gsc_label("Overall AI Rate");
			gsc_label_long_comma(total);
			printf(" S/S\n");
			gsc_label_level_dec();
		}
	}

	if (fsamp)
		fsamp[0]	= total;

	return(errs);
}


