// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/utils/util_id.c $
// $Rev: 53259 $
// $Date: 2023-06-22 16:00:16 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Utilities Library: source file

#include "main.h"



//*****************************************************************************
static int _id_device_pci(int fd)
{
	int	errs	= 0;
	u32	reg;

	gsc_label("Vendor ID");
	errs	+= aio_reg_read(fd, -1, 0, GSC_PCI_9080_VIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Device ID");
	errs	+= aio_reg_read(fd, -1, 0, GSC_PCI_9080_DIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x9080)
	{
		printf("(PCI9080)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Sub Vendor ID");
	errs	+= aio_reg_read(fd, -1, 0, GSC_PCI_9080_SVID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Subsystem ID");
	errs	+= aio_reg_read(fd, -1, 0, GSC_PCI_9080_SID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x2402)
	{
		printf("(16AIO)\n");
	}
	else if (reg == 0x2409)
	{
		printf("(12AIO)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	return(errs);
}



//*****************************************************************************
static int _id_device_frr(int fd)
{
	int	errs;
	u32	reg;

	gsc_label("Firmware Rev Reg");
	errs	= aio_reg_read(fd, -1, 0, AIO_GSC_FRR, &reg);
	printf("0x%08lX\n", (long) reg);

	return(errs);
}



/******************************************************************************
*
*	Function:	aio_id_device
*
*	Purpose:
*
*		Identify the device.
*
*	Arguments:
*
*		fd		The handle to use to access the device.
*
*		index	The index of the device or -1 if we don't care.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio_id_device(int fd, int index, int verbose)
{
	s32			device_type;
	int			errs;
	const char*	psz			= "";

	if (verbose)
		gsc_label_index("Device", index);

	errs	= aio_query(fd, -1, 0, AIO_QUERY_DEVICE_TYPE, &device_type);

	if (errs == 0)
	{
		switch (device_type)
		{
			default:

				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (unexpected device type: %ld)",
							(long) device_type);
				}

				break;

			case GSC_DEV_TYPE_12AIO:

				psz	= "12AIO";
				break;

			case GSC_DEV_TYPE_16AIO:

				psz	= "16AIO";
				break;
		}
	}

	if (verbose)
		printf("%s\n", psz);

	gsc_label_level_inc();

	errs	+= _id_device_pci(fd);
	errs	+= _id_device_frr(fd);

	gsc_label_level_dec();

	return(errs);
}



