// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/utils/util_irq0_sel.c $
// $Rev: 53259 $
// $Date: 2023-06-22 16:00:16 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Utilities Library: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio_irq0_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the AIO_IOCTL_IRQ0_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aio_irq0_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("IRQ 0 Select", index);

	ret		= aio_ioctl(fd, AIO_IOCTL_IRQ0_SEL, &set);
	errs	= ret ? 1 : 0;


	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AIO_IRQ0_IDLE_INIT:

			ptr	= "Idle/Init Complete";
			break;

		case AIO_IRQ0_AUTOCAL_DONE:

			ptr	= "Autocal Done";
			break;

		case AIO_IRQ0_AUX_IN_L2H:

			ptr	= "Aux In Low-to-High";
			break;

		case AIO_IRQ0_AUX_IN_H2L:

			ptr	= "Aux In High-to-Low";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


