// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/utils/util_range.c $
// $Rev: 44027 $
// $Date: 2018-11-06 15:20:54 -0600 (Tue, 06 Nov 2018) $

// 16AIO: Utilities Library: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio_range
*
*	Purpose:
*
*		Provide a visual wrapper for the AIO_IOCTL_RANGE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aio_range(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Voltage Range", index);

	ret		= aio_ioctl(fd, AIO_IOCTL_RANGE, &set);
	errs	= ret ? 1 : 0;


	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AIO_RANGE_2_5V:

			ptr	= "+- 2.5 Volts";
			break;

		case AIO_RANGE_5V:

			ptr	= "+- 5 Volts";
			break;

		case AIO_RANGE_10V:

			ptr	= "+- 10 Volts";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


