// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/utils/util_reg.c $
// $Rev: 53259 $
// $Date: 2023-06-22 16:00:16 -0500 (Thu, 22 Jun 2023) $

// 16AIO: Utilities Library: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)			"GSC " #a, AIO_GSC_##a, 0, 0
#define	WIDTH				24
#define	SIZEOF_ARRAY(a)		(sizeof((a)) / sizeof((a)[0]))



//*****************************************************************************
static int _bcr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	aim[]	=
	{
		"Differential",
		"Single-Ended",
		"Zero Test",
		"+Vref Test",
		"Output Channel 0 Loopback",
		"Output Channel 1 Loopback",
		"Output Channel 2 Loopback",
		"Output Channel 3 Loopback"
	};

	static const char*	range[]	=
	{
		"+-2.5 Volts",
		"+-5 Volts",
		"+-10 Volts",
		"+-10 Volts"
	};

	static const char*	disabled[]	= { "Disabled",			"Enabled"			};
	static const char*	fail[]		= { "No (fail)",		"Yes (pass)"		};
	static const char*	idle[]		= { "Idle",				"Active"			};
	static const char*	seq[]		= { "No (Sequential)",	"Yes (Synchronous)"	};
	static const char*	twos[]		= { "Twos Compliment",	"Offset Binary"		};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value,  2,  0, 1, aim,			"Input Mode"			);
	gsc_reg_field_show(width, WIDTH, value,  3,  3, 1, NULL,		"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value,  5,  4, 1, range,		"Range"					);
	gsc_reg_field_show(width, WIDTH, value,  6,  6, 1, twos,		"Offset Binary"			);
	gsc_reg_field_show(width, WIDTH, value,  7,  7, 1, NULL,		"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value,  8,  8, 1, seq,			"Simultaneous Outputs"	);
	gsc_reg_field_show(width, WIDTH, value,  9,  9, 1, disabled,	"Output Burst"			);
	gsc_reg_field_show(width, WIDTH, value, 10, 10, 1, disabled,	"Output Looping"		);
	gsc_reg_field_show(width, WIDTH, value, 11, 11, 1, idle,		"Output Sync"			);
	gsc_reg_field_show(width, WIDTH, value, 12, 12, 1, idle,		"Input Sync"			);
	gsc_reg_field_show(width, WIDTH, value, 13, 13, 1, idle,		"Autocal"				);
	gsc_reg_field_show(width, WIDTH, value, 14, 14, 1, fail,		"Autocal Pass"			);
	gsc_reg_field_show(width, WIDTH, value, 15, 15, 1, idle,		"Initialize"			);
	gsc_reg_field_show(width, WIDTH, value, 31, 16, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _icr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	irq0[]	=
	{
		"Idle/Initialize Done",
		"Autocalibration Done",
		"Auxiliary Input Low-to-High",
		"Auxiliary Input High-to-Low",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	irq1[]	=
	{
		"Idle/Disable",
		"Input Buffer Threshold Low-to-High",
		"Input Buffer Threshold High-to-Low",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	irq2[]	=
	{
		"Idle/Disable",
		"Output Buffer Threshold Low-to-High",
		"Output Buffer Threshold High-to-Low",
		"Output Burst Completed",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	idle[]	= { "Idle",	"Active"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value,  2,  0, 1, irq0,	"IRQ0 Select"	);
	gsc_reg_field_show(width, WIDTH, value,  3,  3, 1, idle,	"IRQ0 Status"	);
	gsc_reg_field_show(width, WIDTH, value,  6,  4, 1, irq1,	"IRQ1 Select"	);
	gsc_reg_field_show(width, WIDTH, value,  7,  7, 1, idle,	"IRQ1 Status"	);
	gsc_reg_field_show(width, WIDTH, value, 10,  8, 1, irq2,	"IRQ2 Select"	);
	gsc_reg_field_show(width, WIDTH, value, 11, 11, 1, idle,	"IRQ2 Status"	);
	gsc_reg_field_show(width, WIDTH, value, 31, 12, 1, NULL,	"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _idbr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	idle[]	= { "Idle",	"Active"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 15,  0, 1, NULL,	"Data"		);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, idle,	"Tag"		);
	gsc_reg_field_show(width, WIDTH, value, 31, 17, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _ibcr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	idle[]	= { "Idle",	"Active"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 14,  0, 1, NULL,	"Threshold"			);
	gsc_reg_field_show(width, WIDTH, value, 15, 15, 1, idle,	"Clear Buffer"		);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, idle,	"Threshold Flag"	);
	gsc_reg_field_show(width, WIDTH, value, 31, 17, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _rgxr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	enabled[]	= { "Enabled",	"Disabled"	};

	long	v;

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 15,  0, 0, NULL,	"Nrate"				);
	v	= value & 0xFFFF;
	printf("0x%lX  (Fref / %ld)\n", v, v);

	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, enabled,	"Generator Disable"	);
	gsc_reg_field_show(width, WIDTH, value, 31, 17, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _odbr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	idle[]	= { "Idle",	"Active"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 15,  0, 1, NULL,	"Data"					);
	gsc_reg_field_show(width, WIDTH, value, 17, 16, 1, NULL,	"Output Channel Tag"	);
	gsc_reg_field_show(width, WIDTH, value, 18, 18, 1, idle,	"Group End Flag"		);
	gsc_reg_field_show(width, WIDTH, value, 19, 19, 1, idle,	"Burst End Flag"		);
	gsc_reg_field_show(width, WIDTH, value, 31, 20, 1, NULL,	"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _obcr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	idle[]	= { "Idle",	"Active"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 14,  0, 1, NULL,	"Threshold Level"	);
	gsc_reg_field_show(width, WIDTH, value, 15, 15, 1, idle,	"Clear Buffer"		);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, idle,	"Threshold Flag"	);
	gsc_reg_field_show(width, WIDTH, value, 31, 17, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _sscr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	ext_sync[]	=
	{
		"Input Scan Clock",
		"Output Sync",
		"Ext Sync Input",
		"Disabled"
	};

	static const char*	in_clk[]	=
	{
		"Rate Gen A",
		"Rate Gen B",
		"Ext Sync Input",
		"BCR Input Sync"
	};

	static const char*	out_clk[]	=
	{
		"Rate Gen A",
		"Rate Gen B",
		"Ext Sync Input",
		"Disabled"
	};

	static const char*	out_sync[]	=
	{
		"Rate Gen A",
		"Rate Gen B",
		"Ext Sync Input",
		"BCR Output Sync"
	};

	static const char*	size[]	=
	{
		"4 Channels (0-3)",
		"8 Channels (0-7)",
		"16 Channels (0-15)",
		"32 Channels (0-31)"
	};

	static const char*	master[]	= { "Master",	"Rate Gen A"	};
	static const char*	multi[]		= { "Multi",	"Single"		};
	static const char*	no[]		= { "No",		"Yes"			};

	int	chans;
	u32	field;

	width	+= 10;

	if (value & 0x20000)
		chans	= 2;
	else if (value & 0x800)
		chans	= 1;
	else
		chans	= 3;

	gsc_reg_field_show(width, WIDTH, value,  1,  0, 0, size,		"Scan Size"				);
	printf("%s\n", (chans == 3) ? "" : "  (ignore)");

	gsc_reg_field_show(width, WIDTH, value,  3,  2, 1, in_clk,		"Input Clock Src"		);
	gsc_reg_field_show(width, WIDTH, value,  5,  4, 1, out_clk,		"Output Clock Src"		);
	gsc_reg_field_show(width, WIDTH, value,  7,  6, 1, out_sync,	"Output Sync Src"		);
	gsc_reg_field_show(width, WIDTH, value,  9,  8, 1, ext_sync,	"Ext Sync Output Src"	);
	gsc_reg_field_show(width, WIDTH, value, 10, 10, 1, master,		"Rate Gen B Source"		);

	gsc_reg_field_show(width, WIDTH, value, 11, 11, 0, multi,		"Input Scanning Mode"	);
	printf("%s\n", (chans == 2) ? "  (ignore)" : "");

	gsc_reg_field_show(width, WIDTH, value, 16, 12, 0, NULL,		"Single-Channel Select"	);
	field	= (value >> 12) & 0x1F;
	printf("Channel # %ld%s\n", (long) field, (chans == 1) ? "" : "  (ignore)");

	gsc_reg_field_show(width, WIDTH, value, 17, 17, 1, no,			"Two Channel Scan"		);
	gsc_reg_field_show(width, WIDTH, value, 31, 18, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _diopr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	dir[]	= { "No (Input)",	"Yes (Output)"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value,  7,  0, 1, NULL,	"Low Byte"		);
	gsc_reg_field_show(width, WIDTH, value, 15,  8, 1, NULL,	"High Byte"		);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, NULL,	"Aux Input"		);
	gsc_reg_field_show(width, WIDTH, value, 17, 17, 1, NULL,	"Aux Output"	);
	gsc_reg_field_show(width, WIDTH, value, 18, 18, 1, dir,		"Low Byte Out"	);
	gsc_reg_field_show(width, WIDTH, value, 19, 19, 1, dir,		"High Byte Out"	);
	gsc_reg_field_show(width, WIDTH, value, 31, 20, 1, NULL,	"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _frr_detail(int fd, int supported, u32 value, int width)
{
	u32	field;

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 11,  0, 0, NULL,	"Firmware Revision"	);
	field	= value & 0xFFF;
	printf("%03lX\n", (long) field);

	gsc_reg_field_show(width, WIDTH, value, 31, 12, 1, NULL,	"Reserved"			);
	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BCR),	0,	_bcr_detail,	"Board Control Register"			},
	{ _GSC_REG(ICR),	0,	_icr_detail,	"Interrupt Control Register"		},
	{ _GSC_REG(IDBR),	0,	_idbr_detail,	"Input Data Buffer Register"		},
	{ _GSC_REG(IBCR),	0,	_ibcr_detail,	"Input Buffer Control Register"		},
	{ _GSC_REG(RGAR),	0,	_rgxr_detail,	"Rate Generator A Register"			},
	{ _GSC_REG(RGBR),	0,	_rgxr_detail,	"Rate Generator B Register"			},
	{ _GSC_REG(ODBR),	0,	_odbr_detail,	"Output Data Buffer Register"		},
	{ _GSC_REG(OBCR),	0,	_obcr_detail,	"Output Buffer Control Register"	},
	{ _GSC_REG(SSCR),	0,	_sscr_detail,	"Scan and Sync Control Register"	},
	{ _GSC_REG(DIOPR),	0,	_diopr_detail,	"Digital I/O Port Register"			},
	{ _GSC_REG(FRR),	0,	_frr_detail,	"Firmware Revision Register"		},
	{ _GSC_REG(AVR),	0,	NULL,			"Autocal Values Register"			},
	{ NULL, 0, 0, 0,	0,	NULL,			NULL								}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	aio_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* aio_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	aio_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* aio_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	aio_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* aio_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	aio_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* aio_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	aio_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, aio_reg_read);
	return(errs);
}



